@@ -1,7 +1,7 @@
 
 /* Generated data (by glib-mkenums) */
 
-#ifndef __GDK_PIXBUF__ENUM_TYPES_H__
+#ifndef __GDK_PIXBUF_ENUM_TYPES_H__
 #define __GDK_PIXBUF_ENUM_TYPES_H__
 
 /* enumerations from "gdk-pixbuf.h" */
