@@ -25,17 +25,18 @@
  */
 
 #include <stdio.h>
+
 #include "gdktypes.h"
 #include "gdkprivate.h"
 #include "config.h"
 
-guint             gdk_debug_flags = 0;
-GdkWindow        *gdk_parent_root = NULL;
-gint              gdk_error_code = 0;
-gint              gdk_error_warnings = TRUE;
-GList            *gdk_default_filters = NULL;
-GMutex           *gdk_threads_mutex = NULL;          /* Global GDK lock */
-GList            *gdk_queued_events = NULL;
-GList            *gdk_queued_tail = NULL;
+guint             _gdk_debug_flags = 0;
+GdkWindow        *_gdk_parent_root = NULL;
+gint              _gdk_error_code = 0;
+gint              _gdk_error_warnings = TRUE;
+GList            *_gdk_default_filters = NULL;
+GList            *_gdk_queued_events = NULL;
+GList            *_gdk_queued_tail = NULL;
 
+GDKVAR GMutex     *gdk_threads_mutex = NULL;          /* Global GDK lock */
 
