@@ -67,7 +67,8 @@ typedef enum {
   GDK_DEBUG_MISC          = 1 << 0,
   GDK_DEBUG_EVENTS        = 1 << 1,
   GDK_DEBUG_DND           = 1 << 2,
-  GDK_DEBUG_XIM           = 1 << 3
+  GDK_DEBUG_XIM           = 1 << 3,
+  GDK_DEBUG_NOGRABS       = 1 << 4,
 } GdkDebugFlag;
 
 #ifndef GDK_DISABLE_DEPRECATED
