@@ -184,7 +184,7 @@ _gdk_fb_get_image (GdkDrawable *drawable,
   image->visual = gdk_drawable_get_visual (drawable);
   image->width = width;
   image->height = height;
-  image->bits_per_pixel = GDK_DRAWABLE_IMPL_FBDATA (gdk_parent_root)->depth;
+  image->bits_per_pixel = GDK_DRAWABLE_FBDATA (drawable)->depth;
   image->depth = image->bits_per_pixel;
 
   if (image->bits_per_pixel <= 8)
@@ -210,7 +210,7 @@ _gdk_fb_get_image (GdkDrawable *drawable,
   fbd.drawable_data.depth = image->depth;
   fbd.drawable_data.window_type = GDK_DRAWABLE_PIXMAP;
 
-  if (GDK_DRAWABLE_FBDATA (drawable)->mem == GDK_DRAWABLE_IMPL_FBDATA (gdk_parent_root)->mem)
+  if (GDK_DRAWABLE_FBDATA (drawable)->mem == GDK_DRAWABLE_IMPL_FBDATA (_gdk_parent_root)->mem)
     {
       region = gdk_fb_clip_region (drawable, _gdk_fb_screen_gc, TRUE, FALSE, FALSE);
 
@@ -311,6 +311,6 @@ gdk_image_put_pixel (GdkImage *image,
 }
 
 void
-gdk_image_exit(void)
+_gdk_image_exit(void)
 {
 }
