@@ -31,7 +31,7 @@
 #include <pango/pangowin32.h>
 
 #include "gdkfont.h"
-#include "gdkinternals.h"
+#include "gdkpango.h" /* gdk_pango_context_get() */
 #include "gdkprivate-win32.h"
 
 static GHashTable *font_name_hash = NULL;
@@ -1520,7 +1520,6 @@ gdk_font_load (const gchar *font_name)
   GdkFontPrivateWin32 *private;
   GdkWin32SingleFont *singlefont;
   HGDIOBJ oldfont;
-  HANDLE *f;
   TEXTMETRIC textmetric;
 
   g_return_val_if_fail (font_name != NULL, NULL);
@@ -1580,7 +1579,7 @@ gdk_font_from_description (PangoFontDescription *font_desc)
   g_return_val_if_fail (font_desc != NULL, NULL);
 
   font_map = pango_win32_font_map_for_display ();
-  font = pango_font_map_load_font (font_map, font_desc);
+  font = pango_font_map_load_font (font_map, gdk_pango_context_get (), font_desc);
 
   if (font)
     {
@@ -1727,7 +1726,7 @@ gdk_font_id (const GdkFont *font)
     return 0;
 }
 
-gint
+gboolean
 gdk_font_equal (const GdkFont *fonta,
                 const GdkFont *fontb)
 {
@@ -1791,7 +1790,7 @@ unicode_classify (wchar_t wc)
 }
 
 void
-gdk_wchar_text_handle (GdkFont       *font,
+_gdk_wchar_text_handle (GdkFont       *font,
 		       const wchar_t *wcstr,
 		       int            wclen,
 		       void         (*handler)(GdkWin32SingleFont *,
@@ -1812,7 +1811,7 @@ gdk_wchar_text_handle (GdkFont       *font,
 
   g_assert (private->base.ref_count > 0);
 
-  GDK_NOTE (MISC, g_print ("gdk_wchar_text_handle: "));
+  GDK_NOTE (MISC, g_print ("_gdk_wchar_text_handle: "));
 
   while (wcp < end)
     {
@@ -1901,14 +1900,14 @@ gdk_text_size (GdkFont           *font,
       /* For single characters, don't try to interpret as UTF-8.
        */
       wcstr[0] = (guchar) text[0];
-      gdk_wchar_text_handle (font, wcstr, 1, gdk_text_size_handler, arg);
+      _gdk_wchar_text_handle (font, wcstr, 1, gdk_text_size_handler, arg);
     }
   else
     {
-      if ((wlen = gdk_nmbstowchar_ts (wcstr, text, text_length, text_length)) == -1)
-	g_warning ("gdk_text_size: gdk_nmbstowchar_ts failed");
+      if ((wlen = _gdk_win32_nmbstowchar_ts (wcstr, text, text_length, text_length)) == -1)
+	g_warning ("gdk_text_size: _gdk_win32_nmbstowchar_ts failed");
       else
-	gdk_wchar_text_handle (font, wcstr, wlen, gdk_text_size_handler, arg);
+	_gdk_wchar_text_handle (font, wcstr, wlen, gdk_text_size_handler, arg);
     }
 
   g_free (wcstr);
@@ -1959,7 +1958,7 @@ gdk_text_width_wc (GdkFont	  *font,
 
   arg.total.cx = arg.total.cy = 0;
 
-  gdk_wchar_text_handle (font, wcstr, text_length,
+  _gdk_wchar_text_handle (font, wcstr, text_length,
 			 gdk_text_size_handler, &arg);
 
   if (sizeof (wchar_t) != sizeof (GdkWChar))
@@ -2008,14 +2007,14 @@ gdk_text_extents (GdkFont     *font,
   if (text_length == 1)
     {
       wcstr[0] = (guchar) text[0];
-      gdk_wchar_text_handle (font, wcstr, 1, gdk_text_size_handler, &arg);
+      _gdk_wchar_text_handle (font, wcstr, 1, gdk_text_size_handler, &arg);
     }
   else
     {
-      if ((wlen = gdk_nmbstowchar_ts (wcstr, text, text_length, text_length)) == -1)
-	g_warning ("gdk_text_extents: gdk_nmbstowchar_ts failed");
+      if ((wlen = _gdk_win32_nmbstowchar_ts (wcstr, text, text_length, text_length)) == -1)
+	g_warning ("gdk_text_extents: _gdk_win32_nmbstowchar_ts failed");
       else
-	gdk_wchar_text_handle (font, wcstr, wlen, gdk_text_size_handler, &arg);
+	_gdk_wchar_text_handle (font, wcstr, wlen, gdk_text_size_handler, &arg);
     }
 
   g_free (wcstr);
@@ -2079,7 +2078,7 @@ gdk_text_extents_wc (GdkFont        *font,
 
   arg.total.cx = arg.total.cy = 0;
 
-  gdk_wchar_text_handle (font, wcstr, text_length,
+  _gdk_wchar_text_handle (font, wcstr, text_length,
 			 gdk_text_size_handler, &arg);
 
   if (sizeof (wchar_t) != sizeof (GdkWChar))
