@@ -263,21 +263,25 @@ gtk_button_box_get_child_property (GtkContainer *container,
 
 /* set per widget values for spacing, child size and child internal padding */
 
-void gtk_button_box_set_child_size (GtkButtonBox *widget, gint width, gint height)
+void 
+gtk_button_box_set_child_size (GtkButtonBox *widget, 
+                               gint width, gint height)
 {
   widget->child_min_width = width;
   widget->child_min_height = height;
 }
 
-void gtk_button_box_set_child_ipadding (GtkButtonBox *widget,
-					gint ipad_x, gint ipad_y)
+void 
+gtk_button_box_set_child_ipadding (GtkButtonBox *widget,
+                                   gint ipad_x, gint ipad_y)
 {
   widget->child_ipad_x = ipad_x;
   widget->child_ipad_y = ipad_y;
 }
 
-void gtk_button_box_set_layout (GtkButtonBox *widget, 
-				GtkButtonBoxStyle layout_style)
+void
+gtk_button_box_set_layout (GtkButtonBox      *widget, 
+                           GtkButtonBoxStyle  layout_style)
 {
   g_return_if_fail (layout_style >= GTK_BUTTONBOX_DEFAULT_STYLE &&
 		    layout_style <= GTK_BUTTONBOX_END);
@@ -293,15 +297,17 @@ void gtk_button_box_set_layout (GtkButtonBox *widget,
 
 /* get per widget values for spacing, child size and child internal padding */
 
-void gtk_button_box_get_child_size (GtkButtonBox *widget,
-				     gint *width, gint *height)
+void 
+gtk_button_box_get_child_size (GtkButtonBox *widget,
+                               gint *width, gint *height)
 {
   *width  = widget->child_min_width;
   *height = widget->child_min_height;
 }
 
-void gtk_button_box_get_child_ipadding (GtkButtonBox *widget,
-					 gint* ipad_x, gint *ipad_y)
+void
+gtk_button_box_get_child_ipadding (GtkButtonBox *widget,
+                                   gint* ipad_x, gint *ipad_y)
 {
   *ipad_x = widget->child_ipad_x;
   *ipad_y = widget->child_ipad_y;
@@ -329,9 +335,9 @@ gtk_button_box_get_layout (GtkButtonBox *widget)
  * is %GTK_BUTTONBOX_START, %GTK_BUTTONBOX_SPREAD or
  * %GTK_BUTTONBOX_EDGE, and before the the other children if the style
  * is %GTK_BUTTONBOX_END. For horizontal button boxes, the definition
- * of before/after depends on direction of the widget. (See
- * gtk_widget_set_direction()) If the style is %GTK_BUTTONBOX_START,
- * or %GTK_BUTTONBOX_START, then the secondary children are aligned at
+ * of before/after depends on direction of the widget (see
+ * gtk_widget_set_direction()). If the style is %GTK_BUTTONBOX_START
+ * or %GTK_BUTTONBOX_END, then the secondary children are aligned at
  * the other end of the button box from the main children. For the
  * other styles, they appear immediately next to the main children.
  **/
