@@ -49,7 +49,9 @@ static void gtk_cell_renderer_pixbuf_render     (GtkCellRenderer            *cel
 
 enum {
 	PROP_ZERO,
-	PROP_PIXBUF
+	PROP_PIXBUF,
+	PROP_PIXBUF_EXPANDER_OPEN,
+	PROP_PIXBUF_EXPANDER_CLOSED
 };
 
 
@@ -104,6 +106,24 @@ gtk_cell_renderer_pixbuf_class_init (GtkCellRendererPixbufClass *class)
 							      GDK_TYPE_PIXBUF,
 							      G_PARAM_READABLE |
 							      G_PARAM_WRITABLE));
+
+	g_object_class_install_property (object_class,
+					 PROP_PIXBUF_EXPANDER_OPEN,
+					 g_param_spec_object ("pixbuf_expander_open",
+							      _("Pixbuf Expander Open"),
+							      _("Pixbuf for open expander."),
+							      GDK_TYPE_PIXBUF,
+							      G_PARAM_READABLE |
+							      G_PARAM_WRITABLE));
+
+	g_object_class_install_property (object_class,
+					 PROP_PIXBUF_EXPANDER_CLOSED,
+					 g_param_spec_object ("pixbuf_expander_closed",
+							      _("Pixbuf Expander Closed"),
+							      _("Pixbuf for closed expander."),
+							      GDK_TYPE_PIXBUF,
+							      G_PARAM_READABLE |
+							      G_PARAM_WRITABLE));
 }
 
 static void
@@ -120,6 +140,14 @@ gtk_cell_renderer_pixbuf_get_property (GObject        *object,
       g_value_set_object (value,
                           cellpixbuf->pixbuf ? G_OBJECT (cellpixbuf->pixbuf) : NULL);
       break;
+    case PROP_PIXBUF_EXPANDER_OPEN:
+      g_value_set_object (value,
+                          cellpixbuf->pixbuf_expander_open ? G_OBJECT (cellpixbuf->pixbuf_expander_open) : NULL);
+      break;
+    case PROP_PIXBUF_EXPANDER_CLOSED:
+      g_value_set_object (value,
+                          cellpixbuf->pixbuf_expander_closed ? G_OBJECT (cellpixbuf->pixbuf_expander_closed) : NULL);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
       break;
@@ -147,6 +175,24 @@ gtk_cell_renderer_pixbuf_set_property (GObject      *object,
       cellpixbuf->pixbuf = pixbuf;
       g_object_notify (object, "pixbuf");
       break;
+    case PROP_PIXBUF_EXPANDER_OPEN:
+      pixbuf = (GdkPixbuf*) g_value_get_object (value);
+      if (pixbuf)
+        g_object_ref (G_OBJECT (pixbuf));
+      if (cellpixbuf->pixbuf_expander_open)
+	g_object_unref (G_OBJECT (cellpixbuf->pixbuf_expander_open));
+      cellpixbuf->pixbuf_expander_open = pixbuf;
+      g_object_notify (object, "pixbuf_expander_open");
+      break;
+    case PROP_PIXBUF_EXPANDER_CLOSED:
+      pixbuf = (GdkPixbuf*) g_value_get_object (value);
+      if (pixbuf)
+        g_object_ref (G_OBJECT (pixbuf));
+      if (cellpixbuf->pixbuf_expander_closed)
+	g_object_unref (G_OBJECT (cellpixbuf->pixbuf_expander_closed));
+      cellpixbuf->pixbuf_expander_closed = pixbuf;
+      g_object_notify (object, "pixbuf_expander_closed");
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
       break;
@@ -182,22 +228,34 @@ gtk_cell_renderer_pixbuf_get_size (GtkCellRenderer *cell,
 				   gint            *height)
 {
   GtkCellRendererPixbuf *cellpixbuf = (GtkCellRendererPixbuf *) cell;
-  GdkPixbuf *pixbuf;
+  gint pixbuf_width = 0;
+  gint pixbuf_height = 0;
   gint calc_width;
   gint calc_height;
 
-  pixbuf = cellpixbuf->pixbuf;
-
-  calc_width = (gint) GTK_CELL_RENDERER (cellpixbuf)->xpad * 2 +
-    (cellpixbuf->pixbuf ? gdk_pixbuf_get_width (cellpixbuf->pixbuf) : 0);
+  if (cellpixbuf->pixbuf)
+    {
+      pixbuf_width = gdk_pixbuf_get_width (cellpixbuf->pixbuf);
+      pixbuf_height = gdk_pixbuf_get_height (cellpixbuf->pixbuf);
+    }
+  if (cellpixbuf->pixbuf_expander_open)
+    {
+      pixbuf_width = MAX (pixbuf_width, gdk_pixbuf_get_width (cellpixbuf->pixbuf_expander_open));
+      pixbuf_height = MAX (pixbuf_height, gdk_pixbuf_get_height (cellpixbuf->pixbuf_expander_open));
+    }
+  if (cellpixbuf->pixbuf_expander_closed)
+    {
+      pixbuf_width = MAX (pixbuf_width, gdk_pixbuf_get_width (cellpixbuf->pixbuf_expander_closed));
+      pixbuf_height = MAX (pixbuf_height, gdk_pixbuf_get_height (cellpixbuf->pixbuf_expander_closed));
+    }
   
-  calc_height = (gint) GTK_CELL_RENDERER (cellpixbuf)->ypad * 2 +
-    (cellpixbuf->pixbuf ? gdk_pixbuf_get_height (cellpixbuf->pixbuf) : 0);
+  calc_width = (gint) GTK_CELL_RENDERER (cellpixbuf)->xpad * 2 + pixbuf_width;
+  calc_height = (gint) GTK_CELL_RENDERER (cellpixbuf)->ypad * 2 + pixbuf_height;
   
   if (x_offset) *x_offset = 0;
   if (y_offset) *y_offset = 0;
 
-  if (cell_area && pixbuf)
+  if (cell_area && pixbuf_width > 0 && pixbuf_height > 0)
     {
       if (x_offset)
 	{
@@ -236,6 +294,15 @@ gtk_cell_renderer_pixbuf_render (GtkCellRenderer    *cell,
   GdkRectangle draw_rect;
 
   pixbuf = cellpixbuf->pixbuf;
+  if (cell->is_expander)
+    {
+      if (cell->is_expanded &&
+	  cellpixbuf->pixbuf_expander_open != NULL)
+	pixbuf = cellpixbuf->pixbuf_expander_open;
+      else if (! cell->is_expanded &&
+	       cellpixbuf->pixbuf_expander_closed != NULL)
+	pixbuf = cellpixbuf->pixbuf_expander_closed;
+    }
 
   if (!pixbuf)
     return;
@@ -248,9 +315,12 @@ gtk_cell_renderer_pixbuf_render (GtkCellRenderer    *cell,
 				     &pix_rect.y,
 				     &pix_rect.width,
 				     &pix_rect.height);
+  
   pix_rect.x += cell_area->x;
   pix_rect.y += cell_area->y;
-
+  pix_rect.width -= cell->xpad * 2;
+  pix_rect.height -= cell->ypad * 2;
+  
   if (gdk_rectangle_intersect (cell_area, &pix_rect, &draw_rect))
     gdk_pixbuf_render_to_drawable_alpha (pixbuf,
                                          window,
