@@ -67,20 +67,23 @@ struct _GtkCheckMenuItemClass
 };
 
 
-GtkType	   gtk_check_menu_item_get_type	       (void) G_GNUC_CONST;
-GtkWidget* gtk_check_menu_item_new	       (void);
-GtkWidget* gtk_check_menu_item_new_with_label  (const gchar	 *label);
-void	   gtk_check_menu_item_set_active      (GtkCheckMenuItem *check_menu_item,
-						gboolean	  is_active);
-void	   gtk_check_menu_item_toggled	       (GtkCheckMenuItem *check_menu_item);
+GtkType	   gtk_check_menu_item_get_type	         (void) G_GNUC_CONST;
+GtkWidget* gtk_check_menu_item_new               (void);
+GtkWidget* gtk_check_menu_item_new_with_label    (const gchar      *label);
+GtkWidget* gtk_check_menu_item_new_with_mnemonic (const gchar      *label);
+void       gtk_check_menu_item_set_active        (GtkCheckMenuItem *check_menu_item,
+                                                  gboolean          is_active);
+gboolean   gtk_check_menu_item_get_active        (GtkCheckMenuItem *check_menu_item);
+void       gtk_check_menu_item_toggled           (GtkCheckMenuItem *check_menu_item);
+void       gtk_check_menu_item_set_inconsistent  (GtkCheckMenuItem *check_menu_item,
+                                                  gboolean          setting);
+gboolean   gtk_check_menu_item_get_inconsistent  (GtkCheckMenuItem *check_menu_item);
 
-void       gtk_check_menu_item_set_inconsistent (GtkCheckMenuItem *check_menu_item,
-                                                 gboolean          setting);
-gboolean   gtk_check_menu_item_get_inconsistent (GtkCheckMenuItem *check_menu_item);
 
 #ifndef GTK_DISABLE_DEPRECATED
 void	   gtk_check_menu_item_set_show_toggle (GtkCheckMenuItem *menu_item,
 						gboolean	  always);
+#define	gtk_check_menu_item_set_state		gtk_check_menu_item_set_active
 #endif
 
 #ifdef __cplusplus
