@@ -42,6 +42,7 @@
 #include "gtkframe.h"
 #include "gtktable.h"
 #include "gtklabel.h"
+#include "gtkmarshalers.h"
 #include "gtkpixmap.h"
 #include "gtkspinbutton.h"
 #include "gtkrange.h"
@@ -143,6 +144,7 @@ static void gtk_color_selection_init		(GtkColorSelection	 *colorsel);
 static void gtk_color_selection_class_init	(GtkColorSelectionClass	 *klass);
 static void gtk_color_selection_destroy		(GtkObject		 *object);
 static void gtk_color_selection_finalize        (GObject		 *object);
+static void gtk_color_selection_realize         (GtkWidget               *widget);
 static void update_color			(GtkColorSelection	 *colorsel);
 static void gtk_color_selection_set_property    (GObject                 *object,
 					         guint                    prop_id,
@@ -162,13 +164,17 @@ static void     gtk_color_selection_set_palette_color   (GtkColorSelection *colo
                                                          GdkColor          *color);
 static void     gtk_color_selection_unset_palette_color (GtkColorSelection *colorsel,
                                                          gint               index);
+static GdkGC   *get_focus_gc                            (GtkWidget         *drawing_area,
+							 gint              *focus_width);
+static void     default_change_palette_func             (const GdkColor    *colors,
+							 gint               n_colors);
 
 static gpointer parent_class = NULL;
 static guint color_selection_signals[LAST_SIGNAL] = { 0 };
 
 static gchar* default_colors = "black:white:gray50:red:purple:blue:light blue:green:yellow:orange:lavender:brown:goldenrod4:dodger blue:pink:light green:gray10:gray30:gray75:gray90";
 
-static GtkColorSelectionChangePaletteFunc change_palette_hook = NULL;
+static GtkColorSelectionChangePaletteFunc change_palette_hook = default_change_palette_func;
 
 static GdkColor current_colors[GTK_CUSTOM_PALETTE_WIDTH * GTK_CUSTOM_PALETTE_HEIGHT];
 
@@ -474,21 +480,23 @@ color_sample_draw_samples (GtkColorSelection *colorsel)
   color_sample_draw_sample (colorsel, 1);
 }
 
-static void
+static gboolean
 color_old_sample_expose (GtkWidget         *da,
 			 GdkEventExpose    *event,
 			 GtkColorSelection *colorsel)
 {
   color_sample_draw_sample (colorsel, 0);
+  return FALSE;
 }
 
 
-static void
+static gboolean
 color_cur_sample_expose (GtkWidget         *da,
 			 GdkEventExpose    *event,
 			 GtkColorSelection *colorsel)
 {
   color_sample_draw_sample (colorsel, 1);
+  return FALSE;
 }
 
 static void
@@ -631,23 +639,45 @@ palette_paint (GtkWidget    *drawing_area,
   
   if (GTK_WIDGET_HAS_FOCUS (drawing_area))
     {
-      GdkGC *gc;
-      gdouble color[4];
-      
-      palette_get_color (drawing_area, color);
-      
-      if (INTENSITY (color[0], color[1], color[2]) > 0.5)
-	gc = drawing_area->style->black_gc;
-      else
-	gc = drawing_area->style->white_gc;
-
+      gint focus_width;
+      GdkGC *gc = get_focus_gc (drawing_area, &focus_width);
       gdk_draw_rectangle (drawing_area->window,
-			  gc, FALSE, 0, 0,
-			  drawing_area->allocation.width - 1,
-			  drawing_area->allocation.height - 1);
+			  gc, FALSE, focus_width / 2, focus_width / 2,
+			  drawing_area->allocation.width - focus_width,
+			  drawing_area->allocation.height - focus_width);
+      g_object_unref (gc);
     }
 }
 
+static GdkGC *
+get_focus_gc (GtkWidget *drawing_area,
+	      gint      *focus_width)
+{
+  GdkGC *gc = gdk_gc_new (drawing_area->window);
+  gdouble color[4];
+  gint8 *dash_list;
+  
+  gtk_widget_style_get (drawing_area,
+			"focus-line-width", focus_width,
+			"focus-line-pattern", (gchar *)&dash_list,
+			NULL);
+      
+  palette_get_color (drawing_area, color);
+      
+  if (INTENSITY (color[0], color[1], color[2]) > 0.5)
+    gdk_gc_copy (gc, drawing_area->style->black_gc);
+  else
+    gdk_gc_copy (gc, drawing_area->style->white_gc);
+
+  gdk_gc_set_line_attributes (gc, *focus_width,
+			      dash_list[0] ? GDK_LINE_ON_OFF_DASH : GDK_LINE_SOLID,
+			      GDK_CAP_BUTT, GDK_JOIN_MITER);
+
+  if (dash_list[0])
+    gdk_gc_set_dashes (gc, 0, dash_list, strlen (dash_list));
+
+  return gc;
+}
 
 static void
 palette_drag_begin (GtkWidget      *widget,
@@ -763,7 +793,6 @@ palette_set_color (GtkWidget         *drawing_area,
 		   gdouble           *color)
 {
   gdouble *new_color = g_new (double, 4);
-  gdouble *old_color;
   GdkColor gdk_color;
   
   gdk_color.red = UNSCALE (color[0]);
@@ -793,31 +822,26 @@ palette_set_color (GtkWidget         *drawing_area,
       
       gtk_object_set_data (GTK_OBJECT (drawing_area), "color_set", GINT_TO_POINTER (1));
     }
-  else
-    {
-      old_color = (gdouble *) gtk_object_get_data (GTK_OBJECT (drawing_area), "color_val");
-      if (old_color)
-	{
-	  g_free (old_color);
-	}
-    }
+
   new_color[0] = color[0];
   new_color[1] = color[1];
   new_color[2] = color[2];
   new_color[3] = 1.0;
   
-  gtk_object_set_data (GTK_OBJECT (drawing_area), "color_val", new_color);
+  g_object_set_data_full (G_OBJECT (drawing_area), "color_val", new_color, (GDestroyNotify)g_free);
 }
 
-static void
+static gboolean
 palette_expose (GtkWidget      *drawing_area,
 		GdkEventExpose *event,
 		gpointer        data)
 {
   if (drawing_area->window == NULL)
-    return;
+    return FALSE;
   
   palette_paint (drawing_area, &(event->area), data);
+  
+  return FALSE;
 }
 
 static void
@@ -1648,10 +1672,11 @@ gtk_color_selection_class_init (GtkColorSelectionClass *klass)
 {
   GtkObjectClass *object_class;
   GObjectClass *gobject_class;
-  gchar *palette;
+  GtkWidgetClass *widget_class;
   
   object_class = GTK_OBJECT_CLASS (klass);
   gobject_class = G_OBJECT_CLASS (klass);
+  widget_class = GTK_WIDGET_CLASS (klass);
   
   parent_class = gtk_type_class (GTK_TYPE_VBOX);
   
@@ -1661,6 +1686,8 @@ gtk_color_selection_class_init (GtkColorSelectionClass *klass)
   gobject_class->set_property = gtk_color_selection_set_property;
   gobject_class->get_property = gtk_color_selection_get_property;
   
+  widget_class->realize = gtk_color_selection_realize;
+
   g_object_class_install_property (gobject_class,
                                    PROP_HAS_OPACITY_CONTROL,
                                    g_param_spec_boolean ("has_opacity_control",
@@ -1695,7 +1722,7 @@ gtk_color_selection_class_init (GtkColorSelectionClass *klass)
                     GTK_RUN_FIRST,
                     GTK_CLASS_TYPE (object_class),
                     GTK_SIGNAL_OFFSET (GtkColorSelectionClass, color_changed),
-                    gtk_marshal_VOID__VOID,
+                    _gtk_marshal_VOID__VOID,
                     GTK_TYPE_NONE, 0);
 
   gtk_settings_install_property (g_param_spec_string ("gtk-color-palette",
@@ -1703,21 +1730,6 @@ gtk_color_selection_class_init (GtkColorSelectionClass *klass)
                                                       _("Palette to use in the color selector"),
                                                       default_colors,
                                                       G_PARAM_READWRITE));
-
-  g_object_get (G_OBJECT (gtk_settings_get_default ()),
-                "gtk-color-palette",
-                &palette,
-                NULL);
-  
-  fill_palette_from_string (palette);
-  g_free (palette);
-
-  change_palette_hook = default_change_palette_func;
-  
-  g_signal_connect_data (G_OBJECT (gtk_settings_get_default ()),
-                         "notify::gtk-color-palette",
-                         G_CALLBACK (palette_change_notify_class),
-                         NULL, NULL, 0);
 }
 
 /* widget functions */
@@ -1742,6 +1754,8 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
   priv->default_alpha_set = FALSE;
   
   priv->tooltips = gtk_tooltips_new ();
+  g_object_ref (priv->tooltips);
+  gtk_object_sink (GTK_OBJECT (priv->tooltips));
   
   gtk_box_set_spacing (GTK_BOX (colorsel), 4);
   top_hbox = gtk_hbox_new (FALSE, 8);
@@ -1875,16 +1889,6 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
   
   gtk_widget_show_all (top_hbox);
 
-  /* Set default colors */
-
-  update_palette (colorsel);
-
-  priv->settings_connection = 
-    g_signal_connect_data (G_OBJECT (gtk_settings_get_default ()),
-                           "notify::gtk-color-palette",
-                           G_CALLBACK (palette_change_notify_instance),
-                           colorsel, NULL, 0);
-  
   /* hide unused stuff */
   
   if (priv->has_opacity == FALSE)
@@ -1914,7 +1918,7 @@ gtk_color_selection_destroy (GtkObject *object)
 
   if (priv->tooltips)
     {
-      gtk_object_destroy (GTK_OBJECT (priv->tooltips));
+      g_object_unref (priv->tooltips);
       priv->tooltips = NULL;
     }
   
@@ -1942,6 +1946,43 @@ gtk_color_selection_finalize (GObject *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+gtk_color_selection_realize (GtkWidget *widget)
+{
+  GtkColorSelection *colorsel = GTK_COLOR_SELECTION (widget);
+  ColorSelectionPrivate *priv = colorsel->private_data;
+  gchar *palette;
+  static gboolean initialized = FALSE;
+
+  if (!initialized)
+    {
+      g_object_get (gtk_settings_get_default (),
+		    "gtk-color-palette", &palette,
+		    NULL);
+      
+      fill_palette_from_string (palette);
+      g_free (palette);
+
+      g_signal_connect (gtk_settings_get_default (),
+			"notify::gtk-color-palette",
+			G_CALLBACK (palette_change_notify_class),
+			NULL);
+
+      initialized = TRUE;
+    }
+  
+  /* Set default colors */
+
+  update_palette (colorsel);
+  priv->settings_connection =
+    g_signal_connect (gtk_settings_get_default (),
+		      "notify::gtk-color-palette",
+		      G_CALLBACK (palette_change_notify_instance),
+		      colorsel);
+
+  if (GTK_WIDGET_CLASS (parent_class)->realize)
+    GTK_WIDGET_CLASS (parent_class)->realize (widget);
+}
 
 /**
  * gtk_color_selection_new:
@@ -1980,17 +2021,15 @@ gtk_color_selection_set_update_policy (GtkColorSelection *colorsel,
 				       GtkUpdateType      policy)
 {
   g_return_if_fail (GTK_IS_COLOR_SELECTION (colorsel));
-  /* */
-  g_warning (G_STRLOC ": This function is deprecated.");
 }
 
 /**
  * gtk_color_selection_get_has_opacity_control:
- * @colorsel: A GtkColorSelection.
+ * @colorsel: a #GtkColorSelection.
  * 
  * Determines whether the colorsel has an opacity control.
  * 
- * Return value: TRUE if the @colorsel has an opacity control.  FALSE if it does't.
+ * Return value: %TRUE if the @colorsel has an opacity control.  %FALSE if it does't.
  **/
 gboolean
 gtk_color_selection_get_has_opacity_control (GtkColorSelection *colorsel)
@@ -2006,8 +2045,8 @@ gtk_color_selection_get_has_opacity_control (GtkColorSelection *colorsel)
 
 /**
  * gtk_color_selection_set_has_opacity_control:
- * @colorsel: A GtkColorSelection.
- * @has_opacity: TRUE if @colorsel can set the opacity, FALSE otherwise.
+ * @colorsel: a #GtkColorSelection.
+ * @has_opacity: %TRUE if @colorsel can set the opacity, %FALSE otherwise.
  *
  * Sets the @colorsel to use or not use opacity.
  * 
@@ -2046,11 +2085,11 @@ gtk_color_selection_set_has_opacity_control (GtkColorSelection *colorsel,
 
 /**
  * gtk_color_selection_get_has_palette:
- * @colorsel: A GtkColorSelection.
+ * @colorsel: a #GtkColorSelection.
  * 
  * Determines whether the color selector has a color palette.
  * 
- * Return value: TRUE if the selector has a palette.  FALSE if it hasn't.
+ * Return value: %TRUE if the selector has a palette.  %FALSE if it hasn't.
  **/
 gboolean
 gtk_color_selection_get_has_palette (GtkColorSelection *colorsel)
@@ -2066,8 +2105,8 @@ gtk_color_selection_get_has_palette (GtkColorSelection *colorsel)
 
 /**
  * gtk_color_selection_set_has_palette:
- * @colorsel: A GtkColorSelection.
- * @has_palette: TRUE if palette is to be visible, FALSE otherwise.
+ * @colorsel: a #GtkColorSelection.
+ * @has_palette: %TRUE if palette is to be visible, %FALSE otherwise.
  *
  * Shows and hides the palette based upon the value of @has_palette.
  * 
@@ -2096,12 +2135,11 @@ gtk_color_selection_set_has_palette (GtkColorSelection *colorsel,
 
 /**
  * gtk_color_selection_set_current_color:
- * @colorsel: A GtkColorSelection.
- * @color: A GdkColor to set the current color with.
+ * @colorsel: a #GtkColorSelection.
+ * @color: A #GdkColor to set the current color with.
  *
  * Sets the current color to be @color.  The first time this is called, it will
  * also set the original color to be @color too.
- * 
  **/
 void
 gtk_color_selection_set_current_color (GtkColorSelection *colorsel,
@@ -2136,12 +2174,11 @@ gtk_color_selection_set_current_color (GtkColorSelection *colorsel,
 
 /**
  * gtk_color_selection_set_current_alpha:
- * @colorsel: A GtkColorSelection.
- * @alpha: an integer between 0 and 65535
+ * @colorsel: a #GtkColorSelection.
+ * @alpha: an integer between 0 and 65535.
  *
  * Sets the current opacity to be @alpha.  The first time this is called, it will
  * also set the original opacity to be @alpha too.
- * 
  **/
 void
 gtk_color_selection_set_current_alpha (GtkColorSelection *colorsel,
@@ -2166,14 +2203,14 @@ gtk_color_selection_set_current_alpha (GtkColorSelection *colorsel,
 
 /**
  * gtk_color_selection_set_color:
- * @colorsel: A GtkColorSelection.
- * @color: A array of doubles that specifies the color to set the current color with.
+ * @colorsel: a #GtkColorSelection.
+ * @color: an array of 4 doubles specifying the red, green, blue and opacity 
+ *   to set the current color to.
  *
  * Sets the current color to be @color.  The first time this is called, it will
  * also set the original color to be @color too.
  *
  * This function is deprecated, use gtk_color_selection_set_current_color() instead.
- *
  **/
 void
 gtk_color_selection_set_color (GtkColorSelection    *colorsel,
@@ -2208,12 +2245,10 @@ gtk_color_selection_set_color (GtkColorSelection    *colorsel,
 
 /**
  * gtk_color_selection_get_current_color:
- * @colorsel: A GtkColorSelection.
- * @color: A GdkColor to fill in with the current color.
+ * @colorsel: a #GtkColorSelection.
+ * @color: a #GdkColor to fill in with the current color.
  *
  * Sets @color to be the current color in the GtkColorSelection widget.
- *
- * This function is deprecated, use gtk_color_selection_get_current_color() instead.
  **/
 void
 gtk_color_selection_get_current_color (GtkColorSelection *colorsel,
@@ -2231,11 +2266,11 @@ gtk_color_selection_get_current_color (GtkColorSelection *colorsel,
 
 /**
  * gtk_color_selection_get_current_alpha:
- * @colorsel: A GtkColorSelection.
+ * @colorsel: a #GtkColorSelection.
  *
- * Returns the current alpha value
+ * Returns the current alpha value.
  *
- * Return value: an integer between 0 and 65535
+ * Return value: an integer between 0 and 65535.
  **/
 guint16
 gtk_color_selection_get_current_alpha (GtkColorSelection *colorsel)
@@ -2250,11 +2285,12 @@ gtk_color_selection_get_current_alpha (GtkColorSelection *colorsel)
 
 /**
  * gtk_color_selection_get_color:
- * @colorsel: A GtkColorSelection.
- * @color: A color to fill in with the current color.
+ * @colorsel: a #GtkColorSelection.
+ * @color: an array of 4 #gdouble to fill in with the current color.
  *
  * Sets @color to be the current color in the GtkColorSelection widget.
  * 
+ * This function is deprecated, use gtk_color_selection_get_current_color() instead.
  **/
 void
 gtk_color_selection_get_color (GtkColorSelection *colorsel,
@@ -2273,14 +2309,13 @@ gtk_color_selection_get_color (GtkColorSelection *colorsel,
 
 /**
  * gtk_color_selection_set_previous_color:
- * @colorsel: A GtkColorSelection.
- * @color: A color to set the previous color with.
+ * @colorsel: a #GtkColorSelection.
+ * @color: a #GdkColor to set the previous color with.
  *
  * Sets the 'previous' color to be @color.  This function should be called with
  * some hesitations, as it might seem confusing to have that color change.
- * Calling gtk_color_selection_set_current_color will also set this color the first
+ * Calling gtk_color_selection_set_current_color() will also set this color the first
  * time it is called.
- * 
  **/
 void
 gtk_color_selection_set_previous_color (GtkColorSelection *colorsel,
@@ -2307,12 +2342,11 @@ gtk_color_selection_set_previous_color (GtkColorSelection *colorsel,
 
 /**
  * gtk_color_selection_set_previous_alpha:
- * @colorsel: A GtkColorSelection.
- * @alpha: an integer between 0 and 65535
+ * @colorsel: a #GtkColorSelection.
+ * @alpha: an integer between 0 and 65535.
  *
  * Sets the 'previous' alpha to be @alpha.  This function should be called with
- * some hesitations, as it might seem confusing to have that color change.
- * 
+ * some hesitations, as it might seem confusing to have that alpha change.
  **/
 void
 gtk_color_selection_set_previous_alpha (GtkColorSelection *colorsel,
@@ -2332,11 +2366,10 @@ gtk_color_selection_set_previous_alpha (GtkColorSelection *colorsel,
 
 /**
  * gtk_color_selection_get_previous_color:
- * @colorsel: A GtkColorSelection.
- * @color: A GdkColor to fill in with the original color value.
+ * @colorsel: a #GtkColorSelection.
+ * @color: a #GdkColor to fill in with the original color value.
  *
  * Fills @color in with the original color value.
- * 
  **/
 void
 gtk_color_selection_get_previous_color (GtkColorSelection *colorsel,
@@ -2354,11 +2387,11 @@ gtk_color_selection_get_previous_color (GtkColorSelection *colorsel,
 
 /**
  * gtk_color_selection_get_previous_alpha:
- * @colorsel: A GtkColorSelection.
+ * @colorsel: a #GtkColorSelection.
  *
- * Returns the previous alpha value
+ * Returns the previous alpha value.
  *
- * Return value: an integer between 0 and 65535
+ * Return value: an integer between 0 and 65535.
  **/
 guint16
 gtk_color_selection_get_previous_alpha (GtkColorSelection *colorsel)
@@ -2373,11 +2406,11 @@ gtk_color_selection_get_previous_alpha (GtkColorSelection *colorsel)
 
 /**
  * gtk_color_selection_set_palette_color:
- * @colorsel: A GtkColorSelection.
- * @index: The color index of the palette.
- * @color: A GdkColor to set the palette with.
+ * @colorsel: a #GtkColorSelection.
+ * @index: the color index of the palette.
+ * @color: A #GdkColor to set the palette with.
  *
- * Set the palette located at at @index to have @color set as its color.
+ * Sets the palette located at @index to have @color as its color.
  * 
  **/
 static void
@@ -2405,14 +2438,14 @@ gtk_color_selection_set_palette_color (GtkColorSelection   *colorsel,
 
 /**
  * gtk_color_selection_get_palette_color:
- * @colorsel: A GtkColorSelection.
- * @index: The color index of the palette.
- * @color: A color to fill in with the color value.
+ * @colorsel: a #GtkColorSelection.
+ * @index: the color index of the palette.
+ * @color: a #GdkColor to fill in with the color value.
  * 
  * Set @color to have the color found in the palette at @index.  If
  * the palette is unset, it will leave the color unset.
  * 
- * Return value: TRUE if the palette located at @index has a color set.  FALSE
+ * Return value: %TRUE if the palette located at @index has a color set.  %FALSE
  * if it doesn't.
  **/
 static gboolean
@@ -2446,11 +2479,10 @@ gtk_color_selection_get_palette_color (GtkColorSelection   *colorsel,
 
 /**
  * gtk_color_selection_unset_palette_color:
- * @colorsel: A GtkColorSelection.
- * @index: The color index in the palette.
+ * @colorsel: a #GtkColorSelection.
+ * @index: the color index in the palette.
  *
- * Change the palette located @index to have no color set.
- * 
+ * Changes the palette located @index to have no color set.
  **/
 static void
 gtk_color_selection_unset_palette_color (GtkColorSelection   *colorsel,
@@ -2471,11 +2503,11 @@ gtk_color_selection_unset_palette_color (GtkColorSelection   *colorsel,
 
 /**
  * gtk_color_selection_get_current_alpha:
- * @colorsel: A GtkColorSelection.
+ * @colorsel: a #GtkColorSelection.
  *
  * Returns the maximum number of palette colors.
  *
- * Return value: the maximum number of palette indexes
+ * Return value: the maximum number of palette indexes.
  **/
 static gint
 gtk_color_selection_get_palette_size (GtkColorSelection *colorsel)
@@ -2486,11 +2518,11 @@ gtk_color_selection_get_palette_size (GtkColorSelection *colorsel)
 
 /**
  * gtk_color_selection_is_adjusting:
- * @colorsel: A GtkColorSelection.
+ * @colorsel: a #GtkColorSelection.
  *
  * Gets the current state of the @colorsel.
  *
- * Return value: TRUE if the user is currently dragging a color around, and FALSE
+ * Return value: %TRUE if the user is currently dragging a color around, and %FALSE
  * if the selection has stopped.
  **/
 gboolean
@@ -2569,14 +2601,14 @@ gtk_color_selection_get_property (GObject     *object,
 
 /**
  * gtk_color_selection_palette_from_string:
- * @str: a string encoding a color palette
- * @colors: return location for allocated array of GdkColor
- * @n_colors: return location for length of array
+ * @str: a string encoding a color palette.
+ * @colors: return location for allocated array of #GdkColor.
+ * @n_colors: return location for length of array.
  * 
  * Parses a color palette string; the string is a colon-separated
  * list of color names readable by gdk_color_parse().
  * 
- * Return value: %TRUE if a palette was successfully parsed
+ * Return value: %TRUE if a palette was successfully parsed.
  **/
 gboolean
 gtk_color_selection_palette_from_string (const gchar *str,
@@ -2655,12 +2687,12 @@ gtk_color_selection_palette_from_string (const gchar *str,
 
 /**
  * gtk_color_selection_palette_to_string:
- * @colors: an array of colors
- * @n_colors: length of the array
+ * @colors: an array of colors.
+ * @n_colors: length of the array.
  * 
  * Encodes a palette as a string, useful for persistent storage.
  * 
- * Return value: allocated string encoding the palette
+ * Return value: allocated string encoding the palette.
  **/
 gchar*
 gtk_color_selection_palette_to_string (const GdkColor *colors,
@@ -2702,14 +2734,14 @@ gtk_color_selection_palette_to_string (const GdkColor *colors,
 
 /**
  * gtk_color_selection_set_change_palette_hook:
- * @func: a function to call when the custom palette needs saving
+ * @func: a function to call when the custom palette needs saving.
  * 
  * Installs a global function to be called whenever the user tries to
  * modify the palette in a color selection. This function should save
  * the new palette contents, and update the GtkSettings property
  * "gtk-color-palette" so all GtkColorSelection widgets will be modified.
  * 
- * Return value: the previous change palette hook (that was replaced)
+ * Return value: the previous change palette hook (that was replaced).
  **/
 GtkColorSelectionChangePaletteFunc
 gtk_color_selection_set_change_palette_hook (GtkColorSelectionChangePaletteFunc func)
@@ -2722,6 +2754,3 @@ gtk_color_selection_set_change_palette_hook (GtkColorSelectionChangePaletteFunc
 
   return old;
 }
-
-
-
