@@ -32,9 +32,6 @@
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
-#ifdef HAVE_DIRENT_H
-#include <dirent.h>
-#endif
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -44,6 +41,9 @@
 #ifdef HAVE_PWD_H
 #include <pwd.h>
 #endif
+#ifdef HAVE_WINSOCK_H
+#include <winsock.h>		/* For gethostname */
+#endif
 
 #include "fnmatch.h"
 
@@ -681,6 +681,10 @@ gtk_file_selection_init (GtkFileSelection *filesel)
   /*  The OK/Cancel button area */
   confirm_area = dialog->action_area;
 
+  /*  The Cancel button  */
+  filesel->cancel_button = gtk_dialog_add_button (dialog,
+                                                  GTK_STOCK_CANCEL,
+                                                  GTK_RESPONSE_CANCEL);
   /*  The OK button  */
   filesel->ok_button = gtk_dialog_add_button (dialog,
                                               GTK_STOCK_OK,
@@ -688,11 +692,6 @@ gtk_file_selection_init (GtkFileSelection *filesel)
   
   gtk_widget_grab_default (filesel->ok_button);
 
-  /*  The Cancel button  */
-  filesel->cancel_button = gtk_dialog_add_button (dialog,
-                                                  GTK_STOCK_CANCEL,
-                                                  GTK_RESPONSE_CANCEL);
-
   /*  The selection entry widget  */
   entry_vbox = gtk_vbox_new (FALSE, 2);
   gtk_box_pack_end (GTK_BOX (filesel->main_vbox), entry_vbox, FALSE, FALSE, 2);
@@ -912,7 +911,7 @@ filenames_drag_get (GtkWidget        *widget,
       else
 	{
 	  g_print ("Setting text: '%s'\n", file);
-	  gtk_selection_data_set_text (selection_data, file);
+	  gtk_selection_data_set_text (selection_data, file, -1);
 	}
     }
 }
@@ -1166,6 +1165,8 @@ gtk_file_selection_finalize (GObject *object)
   GtkFileSelection *filesel = GTK_FILE_SELECTION (object);
 
   g_free (filesel->fileop_file);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 /* Begin file operations callbacks */
@@ -1306,15 +1307,6 @@ gtk_file_selection_create_dir (GtkWidget *widget,
   gtk_widget_show (fs->fileop_entry);
   
   /* buttons */
-  button = gtk_button_new_with_label (_("Create"));
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      (GtkSignalFunc) gtk_file_selection_create_dir_confirmed, 
-		      (gpointer) fs);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
-		     button, TRUE, TRUE, 0);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_show (button);
-  
   button = gtk_button_new_with_label (_("Cancel"));
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     (GtkSignalFunc) gtk_widget_destroy, 
@@ -1325,6 +1317,17 @@ gtk_file_selection_create_dir (GtkWidget *widget,
   gtk_widget_grab_default (button);
   gtk_widget_show (button);
 
+  gtk_widget_grab_focus (fs->fileop_entry);
+
+  button = gtk_button_new_with_label (_("Create"));
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+		      (GtkSignalFunc) gtk_file_selection_create_dir_confirmed, 
+		      (gpointer) fs);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
+		     button, TRUE, TRUE, 0);
+  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+  gtk_widget_show (button);
+  
   gtk_widget_show (dialog);
 }
 
@@ -1432,15 +1435,6 @@ gtk_file_selection_delete_file (GtkWidget *widget,
   g_free (buf);
   
   /* buttons */
-  button = gtk_button_new_with_label (_("Delete"));
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      (GtkSignalFunc) gtk_file_selection_delete_file_confirmed, 
-		      (gpointer) fs);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
-		      button, TRUE, TRUE, 0);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_show (button);
-  
   button = gtk_button_new_with_label (_("Cancel"));
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     (GtkSignalFunc) gtk_widget_destroy, 
@@ -1451,6 +1445,15 @@ gtk_file_selection_delete_file (GtkWidget *widget,
   gtk_widget_grab_default (button);
   gtk_widget_show (button);
 
+  button = gtk_button_new_with_label (_("Delete"));
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+		      (GtkSignalFunc) gtk_file_selection_delete_file_confirmed, 
+		      (gpointer) fs);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
+		      button, TRUE, TRUE, 0);
+  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+  gtk_widget_show (button);
+  
   gtk_widget_show (dialog);
 
 }
@@ -1586,15 +1589,6 @@ gtk_file_selection_rename_file (GtkWidget *widget,
 			      0, strlen (fs->fileop_file));
 
   /* buttons */
-  button = gtk_button_new_with_label (_("Rename"));
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      (GtkSignalFunc) gtk_file_selection_rename_file_confirmed, 
-		      (gpointer) fs);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
-		      button, TRUE, TRUE, 0);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_show (button);
-  
   button = gtk_button_new_with_label (_("Cancel"));
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     (GtkSignalFunc) gtk_widget_destroy, 
@@ -1605,6 +1599,17 @@ gtk_file_selection_rename_file (GtkWidget *widget,
   gtk_widget_grab_default (button);
   gtk_widget_show (button);
 
+  gtk_widget_grab_focus (fs->fileop_entry);
+
+  button = gtk_button_new_with_label (_("Rename"));
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+		      (GtkSignalFunc) gtk_file_selection_rename_file_confirmed, 
+		      (gpointer) fs);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
+		      button, TRUE, TRUE, 0);
+  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+  gtk_widget_show (button);
+  
   gtk_widget_show (dialog);
 }
 
@@ -2584,8 +2589,9 @@ open_new_dir (gchar       *dir_name,
 	      gboolean     stat_subdirs)
 {
   CompletionDirSent *sent;
-  DIR *directory;
-  struct dirent *dirent_ptr;
+  GDir *directory;
+  const char *dirent;
+  GError *error;
   gint entry_count = 0;
   gint n_entries = 0;
   gint i;
@@ -2607,38 +2613,38 @@ open_new_dir (gchar       *dir_name,
       return NULL;
     }
   
-  directory = opendir (sys_dir_name);
+  directory = g_dir_open (sys_dir_name, 0, &error);
   if (!directory)
     {
-      cmpl_errno = errno;
+      cmpl_errno = error->code; /* ??? */
       g_free (sys_dir_name);
       return NULL;
     }
 
-  while ((dirent_ptr = readdir (directory)) != NULL)
+  while ((dirent = g_dir_read_name (directory)) != NULL)
     entry_count++;
 
   sent->entries = g_new (CompletionDirEntry, entry_count);
   sent->entry_count = entry_count;
 
-  rewinddir (directory);
+  g_dir_rewind (directory);
 
   for (i = 0; i < entry_count; i += 1)
     {
-      dirent_ptr = readdir (directory);
+      dirent = g_dir_read_name (directory);
 
-      if (!dirent_ptr)
+      if (!dirent)
 	{
-	  cmpl_errno = errno;
-	  closedir (directory);
+	  g_warning ("Failure reading directory '%s'", sys_dir_name);
+	  g_dir_close (directory);
 	  g_free (sys_dir_name);
 	  return NULL;
 	}
 
-      sent->entries[n_entries].entry_name = g_filename_to_utf8 (dirent_ptr->d_name, -1, NULL, NULL, NULL);
+      sent->entries[n_entries].entry_name = g_filename_to_utf8 (dirent, -1, NULL, NULL, NULL);
       if (!g_utf8_validate (sent->entries[n_entries].entry_name, -1, NULL))
 	{
-	  g_warning (_("The filename %s couldn't be converted to UTF-8. Try setting the environment variable G_BROKEN_FILENAMES."), dirent_ptr->d_name);
+	  g_warning (_("The filename %s couldn't be converted to UTF-8. Try setting the environment variable G_BROKEN_FILENAMES."), dirent);
 	  continue;
 	}
 
@@ -2647,7 +2653,7 @@ open_new_dir (gchar       *dir_name,
 	{
 	  g_string_append_c (path, G_DIR_SEPARATOR);
 	}
-      g_string_append (path, dirent_ptr->d_name);
+      g_string_append (path, dirent);
 
       if (stat_subdirs)
 	{
@@ -2670,7 +2676,7 @@ open_new_dir (gchar       *dir_name,
   g_string_free (path, TRUE);
   qsort (sent->entries, sent->entry_count, sizeof (CompletionDirEntry), compare_cmpl_dir);
 
-  closedir (directory);
+  g_dir_close (directory);
 
   return sent;
 }
