@@ -360,7 +360,8 @@ static void
 gamma_ok_callback (GtkWidget *w, gpointer data)
 {
   GtkGammaCurve *c = data;
-  gchar *start, *end;
+  const gchar *start;
+  gchar *end;
   gfloat v;
 
   start = gtk_entry_get_text (GTK_ENTRY (c->gamma_text));
@@ -399,7 +400,7 @@ button_clicked_callback (GtkWidget *w, gpointer data)
 	  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 2);
 	  gtk_widget_show (hbox);
 	  
-	  label = gtk_label_new (_("_Gamma value"));
+	  label = gtk_label_new_with_mnemonic (_("_Gamma value"));
 	  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
 	  gtk_widget_show (label);
 	  
@@ -412,6 +413,12 @@ button_clicked_callback (GtkWidget *w, gpointer data)
 	  
 	  /* fill in action area: */
 	  hbox = GTK_DIALOG (c->gamma_dialog)->action_area;
+
+	  button = gtk_button_new_with_label (_("Cancel"));
+	  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+			      (GtkSignalFunc) gamma_cancel_callback, c);
+	  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
+	  gtk_widget_show (button);
 	  
 	  button = gtk_button_new_with_label (_("OK"));
 	  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
@@ -421,12 +428,6 @@ button_clicked_callback (GtkWidget *w, gpointer data)
 	  gtk_widget_grab_default (button);
 	  gtk_widget_show (button);
 	  
-	  button = gtk_button_new_with_label (_("Cancel"));
-	  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			      (GtkSignalFunc) gamma_cancel_callback, c);
-	  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
-	  gtk_widget_show (button);
-	  
 	  gtk_widget_show (c->gamma_dialog);
 	}
     }
