@@ -27,6 +27,7 @@
 #include <string.h>
 #include "gtklabel.h"
 #include "gtkmain.h"
+#include "gtkmarshalers.h"
 #include "gtksignal.h"
 #include "gtkwindow.h"
 #include "gdk/gdkkeysyms.h"
@@ -136,9 +137,7 @@ static void gtk_label_hierarchy_changed          (GtkWidget     *widget,
 static void gtk_label_create_window       (GtkLabel *label);
 static void gtk_label_destroy_window      (GtkLabel *label);
 static void gtk_label_clear_layout        (GtkLabel *label);
-static void gtk_label_ensure_layout       (GtkLabel *label,
-                                           gint     *widthp,
-                                           gint     *heightp);
+static void gtk_label_ensure_layout       (GtkLabel *label);
 static void gtk_label_select_region_index (GtkLabel *label,
                                            gint      anchor_index,
                                            gint      end_index);
@@ -258,7 +257,7 @@ gtk_label_class_init (GtkLabelClass *class)
                       GTK_RUN_LAST | GTK_RUN_ACTION,
                       GTK_CLASS_TYPE (object_class),
                       GTK_SIGNAL_OFFSET (GtkLabelClass, move_cursor),
-                      gtk_marshal_VOID__ENUM_INT_BOOLEAN,
+                      _gtk_marshal_VOID__ENUM_INT_BOOLEAN,
                       GTK_TYPE_NONE, 3, GTK_TYPE_MOVEMENT_STEP, GTK_TYPE_INT, GTK_TYPE_BOOL);
   
   signals[COPY_CLIPBOARD] =
@@ -266,7 +265,7 @@ gtk_label_class_init (GtkLabelClass *class)
                     GTK_RUN_LAST | GTK_RUN_ACTION,
                     GTK_CLASS_TYPE (object_class),
                     GTK_SIGNAL_OFFSET (GtkLabelClass, copy_clipboard),
-                    gtk_marshal_VOID__VOID,
+                    _gtk_marshal_VOID__VOID,
                     GTK_TYPE_NONE, 0);
   
   signals[POPULATE_POPUP] =
@@ -274,7 +273,7 @@ gtk_label_class_init (GtkLabelClass *class)
 		    GTK_RUN_LAST,
 		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GtkLabelClass, populate_popup),
-		    gtk_marshal_VOID__OBJECT,
+		    _gtk_marshal_VOID__OBJECT,
 		    GTK_TYPE_NONE, 1, GTK_TYPE_MENU);
 
   g_object_class_install_property (G_OBJECT_CLASS(object_class),
@@ -328,7 +327,7 @@ gtk_label_class_init (GtkLabelClass *class)
                                    g_param_spec_boolean ("wrap",
                                                         _("Line wrap"),
                                                         _("If set, wrap lines if the text becomes too wide."),
-                                                        TRUE,
+                                                        FALSE,
                                                         G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_SELECTABLE,
@@ -580,7 +579,7 @@ gtk_label_init (GtkLabel *label)
   
   label->label = NULL;
 
-  label->jtype = GTK_JUSTIFY_CENTER;
+  label->jtype = GTK_JUSTIFY_LEFT;
   label->wrap = FALSE;
 
   label->use_underline = FALSE;
@@ -601,7 +600,8 @@ gtk_label_init (GtkLabel *label)
  * gtk_label_new:
  * @str: The text of the label
  *
- * Creates a new #GtkLabel, containing the text in @str.
+ * Creates a new label with the given text inside it. You can
+ * pass %NULL to get an empty label widget.
  *
  * Return value: the new #GtkLabel
  **/
@@ -626,9 +626,10 @@ gtk_label_new (const gchar *str)
  * Creates a new #GtkLabel, containing the text in @str.
  *
  * If characters in @str are preceded by an underscore, they are
- * underlined indicating that they represent a keyboard accelerator
- * called a mnemonic.  The mnemonic key can be used to activate
- * another widget, chosen automatically, or explicitly using
+ * underlined. If you need a literal underscore character in a label, use
+ * '__' (two underscores). The first underlined character represents a 
+ * keyboard accelerator called a mnemonic. The mnemonic key can be used 
+ * to activate another widget, chosen automatically, or explicitly using
  * gtk_label_set_mnemonic_widget().
  * 
  * If gtk_label_set_mnemonic_widget()
@@ -666,6 +667,10 @@ gtk_label_mnemonic_activate (GtkWidget *widget,
    * widget's ancestry.
    */
   parent = widget->parent;
+
+  if (parent && GTK_IS_NOTEBOOK (parent))
+    return FALSE;
+  
   while (parent)
     {
       if (GTK_WIDGET_CAN_FOCUS (parent) ||
@@ -780,7 +785,7 @@ gtk_label_get_mnemonic_widget (GtkLabel *label)
  * returns the keyval used for the mnemonic accelerator. If there is no
  * mnemonic set up it returns #GDK_VoidSymbol.
  *
- * Returns: GDK keyval usable for accelerators, or GDK_VoidSymbol
+ * Returns: GDK keyval usable for accelerators, or #GDK_VoidSymbol
  **/
 guint
 gtk_label_get_mnemonic_keyval (GtkLabel *label)
@@ -897,9 +902,10 @@ gtk_label_recalculate (GtkLabel *label)
 /**
  * gtk_label_set_text:
  * @label: a #GtkLabel
- * @str: a string
+ * @str: The text you want to set.
  *
- * Sets the text of the label to @str.
+ * Sets the text within the #GtkLabel widget.  It overwrites any text that
+ * was there before.  
  *
  * This will also clear any previously set mnemonic accelerators.
  **/
@@ -1185,7 +1191,8 @@ gtk_label_set_pattern (GtkLabel	   *label,
  * @jtype: a #GtkJustification
  *
  * Sets the alignment of the lines in the text of the label relative to
- * each other.
+ * each other.  %GTK_JUSTIFY_LEFT is the default value when the
+ * widget is first created with gtk_label_new().
  **/
 void
 gtk_label_set_justify (GtkLabel        *label,
@@ -1210,9 +1217,9 @@ gtk_label_set_justify (GtkLabel        *label,
  * gtk_label_get_justify:
  * @label: a #GtkLabel
  *
- * Returns the justification of the label. See gtk_label_set_justification ().
+ * Returns the justification of the label. See gtk_label_set_justify ().
  *
- * Return value: GtkJustification
+ * Return value: #GtkJustification
  **/
 GtkJustification
 gtk_label_get_justify (GtkLabel *label)
@@ -1227,8 +1234,10 @@ gtk_label_get_justify (GtkLabel *label)
  * @label: a #GtkLabel
  * @wrap: the setting
  *
- * If true, the lines will be wrapped if the text becomes too wide.
- */
+ * Toggles line wrapping within the #GtkLabel widget.  %TRUE makes it break
+ * lines if text exceeds the widget's size.  %FALSE lets the text get cut off
+ * by the edge of the widget if it exceeds the widget size.
+ **/
 void
 gtk_label_set_line_wrap (GtkLabel *label,
 			 gboolean  wrap)
@@ -1319,9 +1328,7 @@ gtk_label_clear_layout (GtkLabel *label)
 }
 
 static void
-gtk_label_ensure_layout (GtkLabel *label,
-                         gint     *widthp,
-                         gint     *heightp)
+gtk_label_ensure_layout (GtkLabel *label)
 {
   GtkWidget *widget;
   PangoRectangle logical_rect;
@@ -1329,27 +1336,6 @@ gtk_label_ensure_layout (GtkLabel *label,
 
   widget = GTK_WIDGET (label);
 
-  /*
-   * There are a number of conditions which will necessitate re-filling
-   * our text:
-   *
-   *     1. text changed.
-   *     2. justification changed either from to to GTK_JUSTIFY_FILL.
-   *     3. font changed.
-   *
-   * These have been detected elsewhere, and label->words will be zero,
-   * if one of the above has occured.
-   *
-   * Additionally, though, if GTK_JUSTIFY_FILL, we need to re-fill if:
-   *
-   *     4. gtk_widget_set_usize has changed the requested width.
-   *     5. gtk_misc_set_padding has changed xpad.
-   *     6.  maybe others?...
-   *
-   * Too much of a pain to detect all these case, so always re-fill.  I
-   * don't think it's really that slow.
-   */
-
   rwidth = label->misc.xpad * 2;
   rheight = label->misc.ypad * 2;
 
@@ -1371,7 +1357,7 @@ gtk_label_ensure_layout (GtkLabel *label,
 	  align = PANGO_ALIGN_RIGHT;
 	  break;
 	case GTK_JUSTIFY_CENTER:
-	  align = PANGO_ALIGN_LEFT;
+	  align = PANGO_ALIGN_CENTER;
 	  break;
 	case GTK_JUSTIFY_FILL:
 	  /* FIXME: This just doesn't work to do this */
@@ -1383,107 +1369,74 @@ gtk_label_ensure_layout (GtkLabel *label,
 	}
 
       pango_layout_set_alignment (label->layout, align);
-    }
 
-  if (label->wrap)
-    {
-      GtkWidgetAuxInfo *aux_info;
-      gint longest_paragraph;
-      gint width, height;
-      gint real_width;
-
-      aux_info = _gtk_widget_get_aux_info (widget, FALSE);
-      if (aux_info && aux_info->width > 0)
+      if (label->wrap)
 	{
-	  pango_layout_set_width (label->layout, aux_info->width * PANGO_SCALE);
-	  pango_layout_get_extents (label->layout, NULL, &logical_rect);
-
-	  rwidth += aux_info->width;
-	  rheight += PANGO_PIXELS (logical_rect.height);
-	}
-      else
-	{
-	  pango_layout_set_width (label->layout, -1);
-	  pango_layout_get_extents (label->layout, NULL, &logical_rect);
-      
-	  width = logical_rect.width;
-	  height = logical_rect.height;
+	  GtkWidgetAuxInfo *aux_info;
+	  gint longest_paragraph;
+	  gint width, height;
 	  
-	  /* Try to guess a reasonable maximum width
-	   */
-	  longest_paragraph = width;
-
-	  width = MIN (width,
-		       PANGO_SCALE * gdk_string_width (gtk_style_get_font (GTK_WIDGET (label)->style),
-						"This long string gives a good enough length for any line to have."));
-	  width = MIN (width,
-		       PANGO_SCALE * (gdk_screen_width () + 1) / 2);
-
-	  pango_layout_set_width (label->layout, width);
-	  pango_layout_get_extents (label->layout, NULL, &logical_rect);
-	  real_width = logical_rect.width;
-	  height = logical_rect.height;
-	  
-	  /* Unfortunately, the above may leave us with a very unbalanced looking paragraph,
-	   * so we try short search for a narrower width that leaves us with the same height
-	   */
-	  if (longest_paragraph > 0)
+	  aux_info = _gtk_widget_get_aux_info (widget, FALSE);
+	  if (aux_info && aux_info->width > 0)
+	    pango_layout_set_width (label->layout, aux_info->width * PANGO_SCALE);
+	  else
 	    {
-	      gint nlines, perfect_width;
+	      pango_layout_set_width (label->layout, -1);
+	      pango_layout_get_extents (label->layout, NULL, &logical_rect);
 
-	      nlines = pango_layout_get_line_count (label->layout);
-	      perfect_width = (longest_paragraph + nlines - 1) / nlines;
+	      width = logical_rect.width;
 	      
-	      if (perfect_width < width)
+	      /* Try to guess a reasonable maximum width */
+	      longest_paragraph = width;
+	      
+	      width = MIN (width,
+			   PANGO_SCALE * gdk_string_width (gtk_style_get_font (GTK_WIDGET (label)->style),
+							   "This long string gives a good enough length for any line to have."));
+	      width = MIN (width,
+			   PANGO_SCALE * (gdk_screen_width () + 1) / 2);
+	      
+	      pango_layout_set_width (label->layout, width);
+	      pango_layout_get_extents (label->layout, NULL, &logical_rect);
+	      height = logical_rect.height;
+	      
+	      /* Unfortunately, the above may leave us with a very unbalanced looking paragraph,
+	       * so we try short search for a narrower width that leaves us with the same height
+	       */
+	      if (longest_paragraph > 0)
 		{
-		  pango_layout_set_width (label->layout, perfect_width);
-		  pango_layout_get_extents (label->layout, NULL, &logical_rect);
+		  gint nlines, perfect_width;
 		  
-		  if (logical_rect.height <= height)
+		  nlines = pango_layout_get_line_count (label->layout);
+		  perfect_width = (longest_paragraph + nlines - 1) / nlines;
+		  
+		  if (perfect_width < width)
 		    {
-		      width = perfect_width;
-		      real_width = logical_rect.width;
-		      height = logical_rect.height;
-		    }
-		  else
-		    {
-		      gint mid_width = (perfect_width + width) / 2;
-
-		      if (mid_width > perfect_width)
+		      pango_layout_set_width (label->layout, perfect_width);
+		      pango_layout_get_extents (label->layout, NULL, &logical_rect);
+		      
+		      if (logical_rect.height <= height)
+			width = perfect_width;
+		      else
 			{
-			  pango_layout_set_width (label->layout, mid_width);
-			  pango_layout_get_extents (label->layout, NULL, &logical_rect);
-
-			  if (logical_rect.height <= height)
+			  gint mid_width = (perfect_width + width) / 2;
+			  
+			  if (mid_width > perfect_width)
 			    {
-			      width = mid_width;
-			      real_width = logical_rect.width;
-			      height = logical_rect.height;
+			      pango_layout_set_width (label->layout, mid_width);
+			      pango_layout_get_extents (label->layout, NULL, &logical_rect);
+			      
+			      if (logical_rect.height <= height)
+				width = mid_width;
 			    }
 			}
 		    }
 		}
+	      pango_layout_set_width (label->layout, width);
 	    }
-	  pango_layout_set_width (label->layout, width);
-
-	  rwidth += PANGO_PIXELS (real_width);
-	  rheight += PANGO_PIXELS (height);
 	}
+      else		/* !label->wrap */
+	pango_layout_set_width (label->layout, -1);
     }
-  else				/* !label->wrap */
-    {
-      pango_layout_set_width (label->layout, -1);
-      pango_layout_get_extents (label->layout, NULL, &logical_rect);
-
-      rwidth += PANGO_PIXELS (logical_rect.width);
-      rheight += PANGO_PIXELS (logical_rect.height);
-    }
-
-  if (widthp)
-    *widthp = rwidth;
-
-  if (heightp)
-    *heightp = rheight;
 }
 
 static void
@@ -1492,13 +1445,44 @@ gtk_label_size_request (GtkWidget      *widget,
 {
   GtkLabel *label;
   gint width, height;
+  PangoRectangle logical_rect;
+  GtkWidgetAuxInfo *aux_info;
   
   g_return_if_fail (GTK_IS_LABEL (widget));
   g_return_if_fail (requisition != NULL);
   
   label = GTK_LABEL (widget);
 
-  gtk_label_ensure_layout (label, &width, &height);
+  /*  
+   * If word wrapping is on, then the height requisition can depend
+   * on:
+   *
+   *   - Any width set on the widget via gtk_widget_set_usize().
+   *   - The padding of the widget (xpad, set by gtk_misc_set_padding)
+   *
+   * Instead of trying to detect changes to these quantities, if we
+   * are wrapping, we just rewrap for each size request. Since
+   * size requisitions are cached by the GTK+ core, this is not
+   * expensive.
+   */
+
+  if (label->wrap)
+    gtk_label_clear_layout (label);
+
+  gtk_label_ensure_layout (label);
+
+  width = label->misc.xpad * 2;
+  height = label->misc.ypad * 2;
+
+  pango_layout_get_extents (label->layout, NULL, &logical_rect);
+  
+  aux_info = _gtk_widget_get_aux_info (widget, FALSE);
+  if (label->wrap && aux_info && aux_info->width > 0)
+    width += aux_info->width;
+  else 
+    width += PANGO_PIXELS (logical_rect.width);
+  
+  height += PANGO_PIXELS (logical_rect.height);
 
   requisition->width = width;
   requisition->height = height;
@@ -1511,7 +1495,7 @@ gtk_label_size_allocate (GtkWidget     *widget,
   GtkLabel *label;
 
   label = GTK_LABEL (widget);
-  
+
   (* GTK_WIDGET_CLASS (parent_class)->size_allocate) (widget, allocation);
 
   if (label->select_info && label->select_info->window)
@@ -1658,7 +1642,7 @@ gtk_label_draw_cursor (GtkLabel  *label, gint xoffset, gint yoffset)
 
       widget_direction = gtk_widget_get_direction (widget);
 
-      gtk_label_ensure_layout (label, NULL, NULL);
+      gtk_label_ensure_layout (label);
       
       pango_layout_get_cursor_pos (label->layout, label->select_info->selection_end,
 				   &strong_pos, &weak_pos);
@@ -1726,7 +1710,7 @@ gtk_label_expose (GtkWidget      *widget,
   
   label = GTK_LABEL (widget);
 
-  gtk_label_ensure_layout (label, NULL, NULL);
+  gtk_label_ensure_layout (label);
   
   if (GTK_WIDGET_VISIBLE (widget) && GTK_WIDGET_MAPPED (widget) &&
       label->text && (*label->text != '\0'))
@@ -1791,10 +1775,10 @@ gtk_label_expose (GtkWidget      *widget,
 	gtk_label_draw_cursor (label, x, y);
     }
 
-  return TRUE;
+  return FALSE;
 }
 
-void
+static void
 gtk_label_set_uline_text_internal (GtkLabel    *label,
 				   const gchar *str)
 {
@@ -2095,7 +2079,7 @@ get_layout_index (GtkLabel *label,
 
   *index = 0;
   
-  gtk_label_ensure_layout (label, NULL, NULL);
+  gtk_label_ensure_layout (label);
   
   window_to_layout_coords (label, &x, &y);
 
@@ -2406,7 +2390,6 @@ get_text_callback (GtkClipboard     *clipboard,
                    gpointer          user_data_or_owner)
 {
   GtkLabel *label;
-  gchar *str;
   
   label = GTK_LABEL (user_data_or_owner);
   
@@ -2430,13 +2413,9 @@ get_text_callback (GtkClipboard     *clipboard,
       if (start > len)
         start = len;
 
-      str = g_strndup (label->text + start,
-                       end - start);
-      
-      gtk_selection_data_set_text (selection_data, 
-                                   str);
-
-      g_free (str);
+      gtk_selection_data_set_text (selection_data,
+				   label->text + start,
+				   end - start);
     }
 }
 
@@ -2452,7 +2431,6 @@ clear_text_callback (GtkClipboard     *clipboard,
     {
       label->select_info->selection_anchor = label->select_info->selection_end;
       
-      gtk_label_clear_layout (label);
       gtk_widget_queue_draw (GTK_WIDGET (label));
     }
 }
@@ -2475,6 +2453,10 @@ gtk_label_select_region_index (GtkLabel *label,
     {
       GtkClipboard *clipboard;
 
+      if (label->select_info->selection_anchor == anchor_index &&
+	  label->select_info->selection_end == end_index)
+	return;
+
       label->select_info->selection_anchor = anchor_index;
       label->select_info->selection_end = end_index;
 
@@ -2495,7 +2477,6 @@ gtk_label_select_region_index (GtkLabel *label,
             gtk_clipboard_clear (clipboard);
         }
 
-      gtk_label_clear_layout (label);
       gtk_widget_queue_draw (GTK_WIDGET (label));
 
       g_object_freeze_notify (G_OBJECT (label));
@@ -2623,7 +2604,7 @@ gtk_label_get_layout (GtkLabel *label)
 {
   g_return_val_if_fail (GTK_IS_LABEL (label), NULL);
 
-  gtk_label_ensure_layout (label, NULL, NULL);
+  gtk_label_ensure_layout (label);
 
   return label->layout;
 }
@@ -2757,7 +2738,7 @@ get_better_cursor (GtkLabel *label,
 		"gtk-split-cursor", &split_cursor,
 		NULL);
 
-  gtk_label_ensure_layout (label, NULL, NULL);
+  gtk_label_ensure_layout (label);
   
   pango_layout_get_cursor_pos (label->layout, index,
 			       &strong_pos, &weak_pos);
@@ -2797,7 +2778,7 @@ gtk_label_move_logically (GtkLabel *label,
       gint n_attrs;
       gint length;
 
-      gtk_label_ensure_layout (label, NULL, NULL);
+      gtk_label_ensure_layout (label);
       
       length = g_utf8_strlen (label->label, -1);
 
@@ -2841,7 +2822,7 @@ gtk_label_move_visually (GtkLabel *label,
       gboolean split_cursor;
       gboolean strong;
 
-      gtk_label_ensure_layout (label, NULL, NULL);
+      gtk_label_ensure_layout (label);
 
       g_object_get (gtk_widget_get_settings (GTK_WIDGET (label)),
 		    "gtk-split-cursor", &split_cursor,
@@ -2895,7 +2876,7 @@ gtk_label_move_forward_word (GtkLabel *label,
       PangoLogAttr *log_attrs;
       gint n_attrs;
 
-      gtk_label_ensure_layout (label, NULL, NULL);
+      gtk_label_ensure_layout (label);
       
       pango_layout_get_log_attrs (label->layout, &log_attrs, &n_attrs);
       
@@ -2926,7 +2907,7 @@ gtk_label_move_backward_word (GtkLabel *label,
       PangoLogAttr *log_attrs;
       gint n_attrs;
 
-      gtk_label_ensure_layout (label, NULL, NULL);
+      gtk_label_ensure_layout (label);
       
       pango_layout_get_log_attrs (label->layout, &log_attrs, &n_attrs);
       
