@@ -84,7 +84,8 @@ static void     gtk_menu_size_request      (GtkWidget        *widget,
 					    GtkRequisition   *requisition);
 static void     gtk_menu_size_allocate     (GtkWidget        *widget,
 					    GtkAllocation    *allocation);
-static void     gtk_menu_paint             (GtkWidget        *widget);
+static void     gtk_menu_paint             (GtkWidget        *widget,
+					    GdkEventExpose   *expose);
 static void     gtk_menu_show              (GtkWidget        *widget);
 static gboolean gtk_menu_expose            (GtkWidget        *widget,
 					    GdkEventExpose   *event);
@@ -953,6 +954,12 @@ gtk_menu_set_tearoff_hints (GtkMenu *menu,
   if (!menu->tearoff_window)
     return;
 
+  if (GTK_WIDGET_VISIBLE (menu->tearoff_scrollbar))
+    {
+      gtk_widget_size_request (menu->tearoff_scrollbar, NULL);
+      width += menu->tearoff_scrollbar->requisition.width;
+    }
+
   geometry_hints.min_width = width;
   geometry_hints.max_width = width;
     
@@ -1053,11 +1060,13 @@ gtk_menu_set_tearoff_state (GtkMenu  *menu,
 	    }
 	  
 	  gtk_menu_reparent (menu, menu->tearoff_hbox, FALSE);
-	  
-	  gdk_window_get_size (GTK_WIDGET (menu)->window, &width, &height);
-	  if (GTK_WIDGET_VISIBLE (menu->tearoff_scrollbar))
-	    width += menu->tearoff_scrollbar->requisition.width;
-	    
+
+	  gdk_window_get_size (GTK_WIDGET (menu)->window, &width, NULL);
+
+	  /* Update menu->requisition
+	   */
+	  gtk_widget_size_request (GTK_WIDGET (menu), NULL);
+  
 	  gtk_menu_set_tearoff_hints (menu, width);
 	    
 	  gtk_widget_realize (menu->tearoff_window);
@@ -1215,8 +1224,6 @@ gtk_menu_realize (GtkWidget *widget)
   gtk_menu_scroll_item_visible (GTK_MENU_SHELL (widget),
 				GTK_MENU_SHELL (widget)->active_menu_item);
 
-  gtk_menu_paint (widget);
-  
   gdk_window_show (menu->bin_window);
   gdk_window_show (menu->view_window);
 }
@@ -1252,7 +1259,6 @@ gtk_menu_size_request (GtkWidget      *widget,
   guint max_toggle_size;
   guint max_accel_width;
   GtkRequisition child_requisition;
-  gint width;
   
   g_return_if_fail (GTK_IS_MENU (widget));
   g_return_if_fail (requisition != NULL);
@@ -1304,15 +1310,10 @@ gtk_menu_size_request (GtkWidget      *widget,
 
   /* If the requested width was different than the allocated width, we need to change
    * the geometry hints for the tear off window so that the window can actually be resized.
-   * Don't resize the tearoff if it is not active, beacuse it won't redraw (it is only a background pixmap).
+   * Don't resize the tearoff if it is not active, because it won't redraw (it is only a background pixmap).
    */
   if ((requisition->width != GTK_WIDGET (menu)->allocation.width) && menu->tearoff_active)
-    {
-      width = requisition->width;
-      if (menu->tearoff_scrollbar && GTK_WIDGET_VISIBLE (menu->tearoff_scrollbar))
-	width += menu->tearoff_scrollbar->requisition.width;
-      gtk_menu_set_tearoff_hints (menu, width);
-    }
+    gtk_menu_set_tearoff_hints (menu, requisition->width);
 }
 
 static void
@@ -1432,9 +1433,8 @@ gtk_menu_size_allocate (GtkWidget     *widget,
 	      
 	      if (!GTK_WIDGET_VISIBLE (menu->tearoff_scrollbar))
 		{
-		  gtk_menu_set_tearoff_hints (menu,
-					      allocation->width + menu->tearoff_scrollbar->requisition.width);
 		  gtk_widget_show (menu->tearoff_scrollbar);
+		  gtk_menu_set_tearoff_hints (menu, allocation->width);
 		  gtk_widget_set_usize (menu->tearoff_window, -1, allocation->height);
 		}
 	    }
@@ -1443,20 +1443,22 @@ gtk_menu_size_allocate (GtkWidget     *widget,
 }
 
 static void
-gtk_menu_paint (GtkWidget *widget)
+gtk_menu_paint (GtkWidget      *widget,
+		GdkEventExpose *event)
 {
-  gint border_x;
-  gint border_y;
-  gint width, height;
-  gint menu_height;
-  gint top_pos;
   GtkMenu *menu;
+  gint width, height;
+  gint border_x, border_y;
   
   g_return_if_fail (GTK_IS_MENU (widget));
 
   menu = GTK_MENU (widget);
   
-  if (GTK_WIDGET_DRAWABLE (widget))
+  border_x = GTK_CONTAINER (widget)->border_width + widget->style->xthickness;
+  border_y = GTK_CONTAINER (widget)->border_width + widget->style->ythickness;
+  gdk_window_get_size (widget->window, &width, &height);
+
+  if (event->window == widget->window)
     {
       gtk_paint_box (widget->style,
 		     widget->window,
@@ -1464,11 +1466,6 @@ gtk_menu_paint (GtkWidget *widget)
 		     GTK_SHADOW_OUT,
 		     NULL, widget, "menu",
 		     0, 0, -1, -1);
-
-      border_x = GTK_CONTAINER (widget)->border_width + widget->style->xthickness;
-      border_y = GTK_CONTAINER (widget)->border_width + widget->style->ythickness;
-      gdk_window_get_size (widget->window, &width, &height);
-
       if (menu->upper_arrow_visible && !menu->tearoff_active)
 	{
 	  gtk_paint_box (widget->style,
@@ -1522,7 +1519,12 @@ gtk_menu_paint (GtkWidget *widget)
 			   MENU_SCROLL_ARROW_HEIGHT - 2 * border_y - 2,
 			   MENU_SCROLL_ARROW_HEIGHT - 2 * border_y - 2);
 	}
-
+    }
+  else if (event->window == menu->view_window)
+    {
+      gint menu_height;
+      gint top_pos;
+      
       if (menu->scroll_offset < 0)
 	gtk_paint_box (widget->style,
 		       menu->view_window,
@@ -1558,7 +1560,7 @@ gtk_menu_expose (GtkWidget	*widget,
 
   if (GTK_WIDGET_DRAWABLE (widget))
     {
-      gtk_menu_paint (widget);
+      gtk_menu_paint (widget, event);
       
       (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
     }
@@ -1658,7 +1660,9 @@ gtk_menu_key_press (GtkWidget	*widget,
       gboolean replace_accels = TRUE;
       const gchar *path;
 
-      path = _gtk_widget_get_accel_path (menu_item);
+      path = GTK_MENU_ITEM (menu_item)->accel_path;
+      if (!path)
+	path = _gtk_widget_get_accel_path (menu_item);
       if (!path)
 	{
 	  /* can't change accelerators on menu_items without paths
@@ -1716,7 +1720,8 @@ gtk_menu_motion_notify  (GtkWidget	   *widget,
    * which may be different from 'widget'.
    */
   menu_item = gtk_get_event_widget ((GdkEvent*) event);
-  if (!menu_item || !GTK_IS_MENU_ITEM (menu_item) || !GTK_WIDGET_IS_SENSITIVE (menu_item) ||
+  if (!menu_item || !GTK_IS_MENU_ITEM (menu_item) ||
+      !_gtk_menu_item_is_selectable (menu_item) ||
       !GTK_IS_MENU (menu_item->parent))
     return FALSE;
 
@@ -2202,14 +2207,8 @@ gtk_menu_position (GtkMenu *menu)
   if (scroll_offset > 0)
     scroll_offset += MENU_SCROLL_ARROW_HEIGHT;
   
-  /* FIXME: The MAX() here is because gtk_widget_set_uposition
-   * is broken. Once we provide an alternate interface that
-   * allows negative values, then we can remove them.
-   */
-  x = MAX (x, 0);
-  gtk_widget_set_uposition (GTK_MENU_SHELL (menu)->active ?
-			    menu->toplevel : menu->tearoff_window, 
-			    x, y);
+  gtk_window_move (GTK_WINDOW (GTK_MENU_SHELL (menu)->active ? menu->toplevel : menu->tearoff_window), 
+		   x, y);
   gtk_widget_set_usize (GTK_MENU_SHELL (menu)->active ?
 			menu->toplevel : menu->tearoff_hbox,
 			-1, requisition.height);
