@@ -26,7 +26,7 @@
  */
 
 #include "gtkmain.h"
-#include "gtkmarshal.h"
+#include "gtkmarshalers.h"
 #include "gtkplug.h"
 #include "gtkprivate.h"
 
@@ -61,7 +61,9 @@ static GdkFilterReturn gtk_plug_filter_func           (GdkXEvent        *gdk_xev
 						       GdkEvent         *event,
 						       gpointer          data);
 
+#if 0
 static void gtk_plug_free_grabbed_keys (GHashTable    *key_table);
+#endif
 static void handle_modality_off        (GtkPlug       *plug);
 static void send_xembed_message        (GtkPlug       *plug,
 					glong          message,
@@ -146,7 +148,7 @@ gtk_plug_class_init (GtkPlugClass *class)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkPlugClass, embedded),
 		  NULL, NULL,
-		  gtk_marshal_VOID__VOID,
+		  _gtk_marshal_VOID__VOID,
 		  GTK_TYPE_NONE, 0);
 }
 
@@ -178,6 +180,14 @@ gtk_plug_set_is_child (GtkPlug  *plug,
 	  plug->modality_group = NULL;
 	}
       
+      /* As a toplevel, the MAPPED flag doesn't correspond
+       * to whether the widget->window is mapped; we unmap
+       * here, but don't bother remapping -- we will get mapped
+       * by gtk_widget_set_parent ().
+       */
+      if (GTK_WIDGET_MAPPED (plug))
+	gtk_widget_unmap (GTK_WIDGET (plug));
+      
       GTK_WIDGET_UNSET_FLAGS (plug, GTK_TOPLEVEL);
       gtk_container_set_resize_mode (GTK_CONTAINER (plug), GTK_RESIZE_PARENT);
 
@@ -216,8 +226,11 @@ _gtk_plug_add_to_socket (GtkPlug   *plug,
 
   gtk_plug_set_is_child (plug, TRUE);
   plug->same_app = TRUE;
+  socket->same_app = TRUE;
   socket->plug_widget = widget;
 
+  plug->socket_window = GTK_WIDGET (socket)->window;
+
   if (GTK_WIDGET_REALIZED (widget))
     gdk_window_reparent (widget->window, plug->socket_window, 0, 0);
 
@@ -227,7 +240,7 @@ _gtk_plug_add_to_socket (GtkPlug   *plug,
 }
 
 /**
- * _gtk_plug_add_to_socket:
+ * _gtk_plug_remove_from_socket:
  * @plug: a #GtkPlug
  * @socket: a #GtkSocket
  * 
@@ -265,7 +278,7 @@ _gtk_plug_remove_from_socket (GtkPlug   *plug,
   socket->same_app = FALSE;
 
   plug->same_app = FALSE;
-  plug->socket_window = FALSE;
+  plug->socket_window = NULL;
 
   gtk_plug_set_is_child (plug, FALSE);
 		    
@@ -409,6 +422,7 @@ gtk_plug_realize (GtkWidget *widget)
   attributes.event_mask = gtk_widget_get_events (widget);
   attributes.event_mask |= (GDK_EXPOSURE_MASK |
 			    GDK_KEY_PRESS_MASK |
+			    GDK_KEY_RELEASE_MASK |
 			    GDK_ENTER_NOTIFY_MASK |
 			    GDK_LEAVE_NOTIFY_MASK |
 			    GDK_FOCUS_CHANGE_MASK |
