@@ -87,6 +87,9 @@ void _gtk_text_btree_tag (const GtkTextIter *start,
 GtkTextLine * _gtk_text_btree_get_line          (GtkTextBTree      *tree,
                                                  gint               line_number,
                                                  gint              *real_line_number);
+GtkTextLine * _gtk_text_btree_get_line_no_last  (GtkTextBTree      *tree,
+                                                 gint               line_number,
+                                                 gint              *real_line_number);
 GtkTextLine * _gtk_text_btree_get_end_iter_line (GtkTextBTree      *tree);
 GtkTextLine * _gtk_text_btree_get_line_at_char  (GtkTextBTree      *tree,
                                                  gint               char_index,
@@ -289,6 +292,10 @@ void _gtk_change_node_toggle_count  (GtkTextBTreeNode   *node,
 void _gtk_text_btree_release_mark_segment (GtkTextBTree       *tree,
                                            GtkTextLineSegment *segment);
 
+/* for coordination with the tag table */
+void _gtk_text_btree_notify_will_remove_tag (GtkTextBTree *tree,
+                                             GtkTextTag   *tag);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
