@@ -36,6 +36,7 @@
 #include "gtksignal.h"
 #include "gtktreedatalist.h"
 #include <string.h>
+#include "gtkintl.h"
 
 typedef struct _SortElt SortElt;
 typedef struct _SortLevel SortLevel;
@@ -62,18 +63,28 @@ struct _SortLevel
 struct _SortData
 {
   GtkTreeModelSort *tree_model_sort;
-  GtkTreePath *parent_a;
-  GtkTreePath *parent_b;
+  GtkTreePath *parent_path;
+  gint parent_path_depth;
+  gint *parent_path_indices;
+  GtkTreeIterCompareFunc sort_func;
+  gpointer sort_data;
 };
 
 struct _SortTuple
 {
   SortElt   *elt;
-  SortLevel *level;
-  SortLevel *children;
   gint       offset;  
 };
 
+/* Properties */
+enum {
+  PROP_0,
+  /* Construct args */
+  PROP_MODEL
+};
+
+
+
 #define GTK_TREE_MODEL_SORT_CACHE_CHILD_ITERS(tree_model_sort) \
 	(((GtkTreeModelSort *)tree_model_sort)->child_flags&GTK_TREE_MODEL_ITERS_PERSIST)
 #define SORT_ELT(sort_elt) ((SortElt *)sort_elt)
@@ -86,6 +97,14 @@ static void gtk_tree_model_sort_class_init            (GtkTreeModelSortClass *tr
 static void gtk_tree_model_sort_tree_model_init       (GtkTreeModelIface     *iface);
 static void gtk_tree_model_sort_tree_sortable_init    (GtkTreeSortableIface  *iface);
 static void gtk_tree_model_sort_finalize              (GObject               *object);
+static void gtk_tree_model_sort_set_property          (GObject         *object,
+						       guint            prop_id,
+						       const GValue    *value,
+						       GParamSpec      *pspec);
+static void gtk_tree_model_sort_get_property          (GObject         *object,
+						       guint            prop_id,
+						       GValue          *value,
+						       GParamSpec      *pspec);
 static void gtk_tree_model_sort_row_changed           (GtkTreeModel          *model,
 						       GtkTreePath           *start_path,
 						       GtkTreeIter           *start_iter,
@@ -203,10 +222,6 @@ static gboolean     gtk_tree_model_sort_insert_value      (GtkTreeModelSort *tre
 							   GtkTreeIter      *s_iter);
 static GtkTreePath *gtk_tree_model_sort_elt_get_path      (SortLevel        *level,
 							   SortElt          *elt);
-static void         get_child_iter_from_elt_no_cache      (GtkTreeModelSort *tree_model_sort,
-							   GtkTreeIter      *child_iter,
-							   SortLevel        *level,
-							   SortElt          *elt);
 static void         get_child_iter_from_elt               (GtkTreeModelSort *tree_model_sort,
 							   GtkTreeIter      *child_iter,
 							   SortLevel        *level,
@@ -217,6 +232,7 @@ static GtkTreePath *gtk_real_tree_model_sort_convert_child_path_to_path (GtkTree
 									 GtkTreePath      *child_path,
 									 gboolean          build_levels);
 
+static GObjectClass *parent_class = NULL;
 
 GType
 gtk_tree_model_sort_get_type (void)
@@ -283,8 +299,21 @@ gtk_tree_model_sort_class_init (GtkTreeModelSortClass *class)
   GObjectClass *object_class;
 
   object_class = (GObjectClass *) class;
+  parent_class = g_type_class_peek_parent (class);
+
+  object_class->set_property = gtk_tree_model_sort_set_property;
+  object_class->get_property = gtk_tree_model_sort_get_property;
 
   object_class->finalize = gtk_tree_model_sort_finalize;
+
+  /* Properties */
+  g_object_class_install_property (object_class,
+                                   PROP_MODEL,
+                                   g_param_spec_object ("model",
+							_("TreeModelSort Model"),
+							_("The model for the TreeModelSort to sort"),
+							GTK_TYPE_TREE_MODEL,
+							G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
@@ -354,6 +383,47 @@ gtk_tree_model_sort_finalize (GObject *object)
       _gtk_tree_data_list_header_free (tree_model_sort->sort_list);
       tree_model_sort->sort_list = NULL;
     }
+
+  /* must chain up */
+  parent_class->finalize (object);
+}
+
+static void
+gtk_tree_model_sort_set_property (GObject      *object,
+				  guint         prop_id,
+				  const GValue *value,
+				  GParamSpec   *pspec)
+{
+  GtkTreeModelSort *tree_model_sort = GTK_TREE_MODEL_SORT (object);
+
+  switch (prop_id)
+    {
+    case PROP_MODEL:
+      gtk_tree_model_sort_set_model (tree_model_sort, g_value_get_object (value));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+static void
+gtk_tree_model_sort_get_property (GObject    *object,
+				  guint       prop_id,
+				  GValue     *value,
+				  GParamSpec *pspec)
+{
+  GtkTreeModelSort *tree_model_sort = GTK_TREE_MODEL_SORT (object);
+
+  switch (prop_id)
+    {
+    case PROP_MODEL:
+      g_value_set_object (value, gtk_tree_model_sort_get_model(tree_model_sort));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
 }
 
 static void
@@ -627,7 +697,6 @@ gtk_tree_model_sort_row_deleted (GtkTreeModel *s_model,
   if (path == NULL)
     return;
 
-  g_print ("path to be deleted: %s\n", gtk_tree_path_to_string (path));
   gtk_tree_model_get_iter (GTK_TREE_MODEL (data), &iter, path);
 
   level = SORT_LEVEL (iter.user_data);
@@ -676,105 +745,70 @@ gtk_tree_model_sort_rows_reordered (GtkTreeModel *s_model,
 				    gint         *new_order,
 				    gpointer      data)
 {
-  int i;
-  int len;
   SortElt *elt;
   SortLevel *level;
-  
-  GtkTreeIter  iter;
+  GtkTreeIter iter;
+  gint *tmp_array;
+  int i, j;
   GtkTreePath *path;
-
   GtkTreeModelSort *tree_model_sort = GTK_TREE_MODEL_SORT (data);
   
-  g_return_if_fail (s_path != NULL && s_iter != NULL);
   g_return_if_fail (new_order != NULL);
 
-  if (!gtk_tree_path_get_indices (s_path))
-    len = gtk_tree_model_iter_n_children (s_model, NULL);
-  else
-    len = gtk_tree_model_iter_n_children (s_model, s_iter);
-
-  if (len < 2)
+  if (s_path == NULL || gtk_tree_path_get_indices (s_path) == NULL)
     {
-      return;
+      if (tree_model_sort->root == NULL)
+	return;
+      path = gtk_tree_path_new ();
+      level = SORT_LEVEL (tree_model_sort->root);
     }
-
-  /** get correct sort level **/
-
-  if (!gtk_tree_path_get_indices (s_path))
-    level = SORT_LEVEL (tree_model_sort->root);
   else
     {
-      path = gtk_tree_model_sort_convert_child_path_to_path (tree_model_sort,
-							     s_path);
-      
-      if (!path)
-	{
-	  return;
-	}
+      path = gtk_real_tree_model_sort_convert_child_path_to_path (tree_model_sort, s_path, FALSE);
+      if (path == NULL)
+	return;
+      gtk_tree_model_get_iter (GTK_TREE_MODEL (data), &iter, path);
 
-      if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (data), &iter, path))
+      level = SORT_LEVEL (iter.user_data);
+      elt = SORT_ELT (iter.user_data2);
+
+      if (!elt->children)
 	{
-	  /* no iter for me */
 	  gtk_tree_path_free (path);
 	  return;
 	}
-      
-      level = SORT_LEVEL (iter.user_data);
-      elt = SORT_ELT (iter.user_data2);
-      gtk_tree_path_free (path);
-
-      /* FIXME: is this needed ? */
-      if (!s_iter)
-	gtk_tree_model_get_iter (s_model, s_iter, s_path);
-
-      if (!elt->children)
-	return;
 
       level = elt->children;
     }
 
-  if (!level)
+  if (level->array->len < 2)
     {
-      /* ignore signal */
+      gtk_tree_path_free (path);
       return;
     }
 
-  if (len != level->array->len)
+  tmp_array = g_new (int, level->array->len);
+  for (i = 0; i < level->array->len; i++)
     {
-      /* length mismatch, pretty bad, shouldn't happen */
-      
-      return;
+      for (j = 0; j < level->array->len; j++)
+	{
+	  if (g_array_index (level->array, SortElt, i).offset == new_order[j])
+	    tmp_array[i] = j;
+	}
     }
-  
-  /** unsorted: set offsets, resort without reordered emission **/
-  if (tree_model_sort->sort_column_id == -1)
+
+  for (i = 0; i < level->array->len; i++)
+    g_array_index (level->array, SortElt, i).offset = tmp_array[i];
+  g_free (tmp_array);
+
+  if (tree_model_sort->sort_column_id == -1 &&
+      tree_model_sort->default_sort_func == (GtkTreeIterCompareFunc) 0x1)
     {
-      path = gtk_tree_model_sort_convert_child_path_to_path (tree_model_sort,
-							     s_path);
-
-      if (!path)
-	{
-	  return;
-	}
-      
-      for (i = 0; i < level->array->len; i++)
-	{
-	  g_array_index (level->array, SortElt, i).offset = new_order[i];
-	  
-	  if (GTK_TREE_MODEL_SORT_CACHE_CHILD_ITERS (tree_model_sort)
-	      && tree_model_sort->sort_column_id == -1)
-	    {
-	      get_child_iter_from_elt_no_cache (tree_model_sort,
-						&(g_array_index (level->array, SortElt, i).iter), level, &g_array_index (level->array, SortElt, i));
-	    }
-	}
-
-      gtk_tree_model_sort_increment_stamp (tree_model_sort);
       
       gtk_tree_model_sort_sort_level (tree_model_sort, level,
 				      FALSE, FALSE);
-
+      gtk_tree_model_sort_increment_stamp (tree_model_sort);
+      
       if (gtk_tree_path_get_depth (path))
 	{
 	  gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_model_sort),
@@ -788,19 +822,9 @@ gtk_tree_model_sort_rows_reordered (GtkTreeModel *s_model,
 	  gtk_tree_model_rows_reordered (GTK_TREE_MODEL (tree_model_sort),
 					 path, NULL, new_order);
 	}
-      
-      gtk_tree_path_free (path);
-      
-      return;
     }
 
-  /** sorted: update offsets, no emission of reordered signal **/
-  for (i = 0; i < level->array->len; i++)
-    g_array_index (level->array, SortElt, i).offset =
-      new_order[g_array_index (level->array, SortElt, i).offset];
-
-  gtk_tree_model_sort_increment_stamp (tree_model_sort);
-
+  gtk_tree_path_free (path);
 }
 
 /* Fulfill our model requirements */
@@ -1290,53 +1314,36 @@ gtk_tree_model_sort_compare_func (gconstpointer a,
 				  gconstpointer b,
 				  gpointer      user_data)
 {
-  gint retval;
-
-  SortElt *sa = ((SortTuple *)a)->elt;
-  SortElt *sb = ((SortTuple *)b)->elt;
-
-  GtkTreeIter iter_a, iter_b;
-
   SortData *data = (SortData *)user_data;
   GtkTreeModelSort *tree_model_sort = data->tree_model_sort;
+  SortTuple *sa = (SortTuple *)a;
+  SortTuple *sb = (SortTuple *)b;
 
-  GtkTreeIterCompareFunc func;
-  gpointer f_data;
+  GtkTreeIter iter_a, iter_b;
+  gint retval;
 
-  if (tree_model_sort->sort_column_id != GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID)
-    {
-      GtkTreeDataSortHeader *header = NULL;
-
-      header = 
-	_gtk_tree_data_list_get_header (tree_model_sort->sort_list,
-					tree_model_sort->sort_column_id);
-      
-      g_return_val_if_fail (header != NULL, 0);
-      g_return_val_if_fail (header->func != NULL, 0);
-      
-      func = header->func;
-      f_data = header->data;
-    }
-  else
-    {
-      /* absolutely SHOULD NOT happen: */
-      g_return_val_if_fail (tree_model_sort->sort_column_id == GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, 0);
-      g_return_val_if_fail (tree_model_sort->default_sort_func != (GtkTreeIterCompareFunc) 0x1, 0);
-      g_return_val_if_fail (tree_model_sort->default_sort_func != NULL, 0);
-
-      func = tree_model_sort->default_sort_func;
-      f_data = tree_model_sort->default_sort_data;
-    }
 
   /* shortcut, if we've the same offsets here, they should be equal */
   if (sa->offset == sb->offset)
     return 0;
-  
-  get_child_iter_from_elt (tree_model_sort, &iter_a, ((SortTuple *)a)->level, sa);
-  get_child_iter_from_elt (tree_model_sort, &iter_b, ((SortTuple *)b)->level, sb);
 
-  retval = (* func) (GTK_TREE_MODEL (tree_model_sort->child_model),
-		     &iter_a, &iter_b, f_data);
+  
+  if (GTK_TREE_MODEL_SORT_CACHE_CHILD_ITERS (tree_model_sort))
+    {
+      iter_a = sa->elt->iter;
+      iter_b = sb->elt->iter;
+    }
+  else
+    {
+      data->parent_path_indices [data->parent_path_depth-1] = sa->elt->offset;
+      gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_model_sort->child_model), &iter_a, data->parent_path);
+      data->parent_path_indices [data->parent_path_depth-1] = sb->elt->offset;
+      gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_model_sort->child_model), &iter_b, data->parent_path);
+    }
+
+  retval = (* data->sort_func) (GTK_TREE_MODEL (tree_model_sort->child_model),
+				&iter_a, &iter_b,
+				data->sort_data);
 
   if (tree_model_sort->order == GTK_SORT_DESCENDING)
     {
@@ -1356,14 +1363,14 @@ gtk_tree_model_sort_offset_compare_func (gconstpointer a,
 {
   gint retval;
 
-  SortElt *sa = ((SortTuple *)a)->elt;
-  SortElt *sb = ((SortTuple *)b)->elt;
+  SortTuple *sa = (SortTuple *)a;
+  SortTuple *sb = (SortTuple *)b;
 
   SortData *data = (SortData *)user_data;
 
-  if (sa->offset < sb->offset)
+  if (sa->elt->offset < sb->elt->offset)
     retval = -1;
-  else if (sa->offset > sb->offset)
+  else if (sa->elt->offset > sb->elt->offset)
     retval = 1;
   else
     retval = 0;
@@ -1393,95 +1400,97 @@ gtk_tree_model_sort_sort_level (GtkTreeModelSort *tree_model_sort,
   GtkTreeIter iter;
   GtkTreePath *path;
   
-  SortData *data;
+  SortData data;
 
   g_return_if_fail (GTK_IS_TREE_MODEL_SORT (tree_model_sort));
   g_return_if_fail (level != NULL);
   
   if (level->array->len < 1 && !((SortElt *)level->array->data)->children)
     return;
-  
-  data = g_new0 (SortData, 1);
 
+  /* Set up data */
+  data.tree_model_sort = tree_model_sort;
   if (level->parent_elt)
     {
-      data->parent_a = gtk_tree_model_sort_elt_get_path (level->parent_level,
-							 level->parent_elt);
-      data->parent_b = gtk_tree_path_copy (data->parent_a);
+      data.parent_path = gtk_tree_model_sort_elt_get_path (level->parent_level,
+							   level->parent_elt);
+      gtk_tree_path_append_index (data.parent_path, 0);
     }
   else
     {
-      data->parent_a = gtk_tree_path_new ();
-      data->parent_b = gtk_tree_path_new ();
+      data.parent_path = gtk_tree_path_new_root ();
     }
+  data.parent_path_depth = gtk_tree_path_get_depth (data.parent_path);
+  data.parent_path_indices = gtk_tree_path_get_indices (data.parent_path);
 
-  data->tree_model_sort = tree_model_sort;
-
+  /* make the array to be sorted */
   sort_array = g_array_sized_new (FALSE, FALSE, sizeof (SortTuple), level->array->len);
-  
   for (i = 0; i < level->array->len; i++)
     {
       SortTuple tuple;
 
       tuple.elt = &g_array_index (level->array, SortElt, i);
-      tuple.level = level;
-      tuple.children = tuple.elt->children;
-      tuple.offset = tuple.elt->offset;
+      tuple.offset = i;
 
       g_array_append_val (sort_array, tuple);
     }
 
-  if (tree_model_sort->sort_column_id == GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID)
-    g_array_sort_with_data (sort_array,
-			    gtk_tree_model_sort_offset_compare_func,
-			    data);
+    if (tree_model_sort->sort_column_id != GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID)
+      {
+	GtkTreeDataSortHeader *header = NULL;
+
+	header = _gtk_tree_data_list_get_header (tree_model_sort->sort_list,
+						 tree_model_sort->sort_column_id);
+      
+	g_return_if_fail (header != NULL);
+	g_return_if_fail (header->func != NULL);
+      
+	data.sort_func = header->func;
+	data.sort_data = header->data;
+      }
+    else
+      {
+	/* absolutely SHOULD NOT happen: */
+	g_return_if_fail (tree_model_sort->default_sort_func != NULL);
+
+	data.sort_func = tree_model_sort->default_sort_func;
+	data.sort_data = tree_model_sort->default_sort_data;
+      }
+
+  if (tree_model_sort->sort_column_id == GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID &&
+      tree_model_sort->default_sort_func == (GtkTreeIterCompareFunc) 0x1)
+    {
+      g_array_sort_with_data (sort_array,
+			      gtk_tree_model_sort_offset_compare_func,
+			      &data);
+    }
   else
-    g_array_sort_with_data (sort_array,
-			    gtk_tree_model_sort_compare_func,
-			    data);
+    {
+      g_array_sort_with_data (sort_array,
+			      gtk_tree_model_sort_compare_func,
+			      &data);
+    }
 
-  gtk_tree_path_free (data->parent_a);
-  gtk_tree_path_free (data->parent_b);
-  g_free (data);
+  gtk_tree_path_free (data.parent_path);
 
-  /* let the world know about our absolutely great new order */
   new_array = g_array_sized_new (FALSE, FALSE, sizeof (SortElt), level->array->len);
-  g_array_set_size (new_array, level->array->len);
   new_order = g_new (gint, level->array->len);
 
   for (i = 0; i < level->array->len; i++)
     {
-      SortElt *elt1;
-      SortElt *elt2;
-      gint j;
+      SortElt *elt;
       
-      elt1 = &g_array_index (level->array, SortElt, i);
+      elt = g_array_index (sort_array, SortTuple, i).elt;
+      new_order[i] = g_array_index (sort_array, SortTuple, i).offset;
 
-      for (j = 0; j < sort_array->len; j++)
-	if (elt1->offset == g_array_index (sort_array, SortTuple, j).offset)
-	  break;
-
-      if (j >= level->array->len)
-	/* isn't supposed to happen */
-	break;
-
-      new_order[i] = j;
-
-      /* copy ... */
-      memcpy (&g_array_index (new_array, SortElt, j), elt1, sizeof (SortElt));
-      elt2 = &g_array_index (new_array, SortElt, j);
-
-      /* point children to correct parent */
-      if (elt2->children)
-	{
-	  elt2->children->parent_elt = elt2;
-	  elt2->children->parent_level = level;
-	}
+      g_array_append_val (new_array, *elt);
+      elt = &g_array_index (new_array, SortElt, i);
+      if (elt->children)
+	elt->children->parent_elt = elt;
     }
 
   g_array_free (level->array, TRUE);
   level->array = new_array;
-  
   g_array_free (sort_array, TRUE);
 
   /* recurse, if possible */
@@ -1500,8 +1509,6 @@ gtk_tree_model_sort_sort_level (GtkTreeModelSort *tree_model_sort,
 
   if (emit_reordered)
     {
-      /* gtk_tree_model_sort_increment_stamp (tree_model_sort); */
-
       if (level->parent_elt)
 	{
 	  iter.stamp = tree_model_sort->stamp;
@@ -1526,7 +1533,7 @@ gtk_tree_model_sort_sort_level (GtkTreeModelSort *tree_model_sort,
     }
   
   g_free (new_order);
-}
+}    
 
 static void
 gtk_tree_model_sort_sort (GtkTreeModelSort *tree_model_sort)
@@ -1710,31 +1717,6 @@ gtk_tree_model_sort_elt_get_path (SortLevel *level,
   return path;
 }
 
-static void
-get_child_iter_from_elt_no_cache (GtkTreeModelSort *tree_model_sort,
-				  GtkTreeIter      *child_iter,
-				  SortLevel        *level,
-				  SortElt          *elt)
-{
-  GtkTreePath *path;
-
-  SortElt *elt_i = elt;
-  SortLevel *level_i = level;
-  
-  path = gtk_tree_path_new ();
-  
-  while (level_i)
-    {
-      gtk_tree_path_prepend_index (path, elt_i->offset);
-      
-      elt_i = level_i->parent_elt;
-      level_i = level_i->parent_level;
-    }
-  
-  gtk_tree_model_get_iter (tree_model_sort->child_model, child_iter, path);
-  gtk_tree_path_free (path);
-}
-
 static void
 get_child_iter_from_elt (GtkTreeModelSort *tree_model_sort,
 			 GtkTreeIter      *child_iter,
@@ -2165,7 +2147,6 @@ gtk_tree_model_sort_free_level (GtkTreeModelSort *tree_model_sort,
 
   g_assert (sort_level);
 
-  g_print ("freeing a level\n");
   if (sort_level->ref_count == 0)
     {
       SortLevel *parent_level = sort_level->parent_level;
